package ejava.corba.regular;

// 
import java.sql.Connection;
import java.sql.Date; // ambiguous with: java.util.Date 
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.Vector;
import java.util.prefs.Preferences;

public class QueryServerImplementation extends QueryServerPOA
{

  private ClientReceiver clientReceiver;
  private String         clientReceiverName;
  private Connection     connection;

  public QueryServerImplementation()
  {

    // Get properties from preferences data
    Preferences preferences =
      Preferences.userNodeForPackage(
        ejava.corba.NamingContextUtility.class);

    // Get connection properties from preferences
    String driverClass     = preferences.get("DRIVER_CLASS", null);
    String dbURL           = preferences.get("DATABASE_URL", null);
    String userName        = preferences.get("USER_NAME", null);
    String password        = preferences.get("PASSWORD", null);
    String otherProperties = preferences.get("SUB_PROPERTIES", null);

    // Load JDBC driver and create connection
    try {
      Class.forName(driverClass);

      if((userName == null) || (password == null)
              || (userName.length() == 0) || (password.length() == 0)) {
        connection = DriverManager.getConnection(dbURL);
      }
      else {
        connection = DriverManager.getConnection(dbURL, userName,
                                                 password);
      }
    } catch(ClassNotFoundException cnfEx) {
      System.out.println("Driver Class Not found " + cnfEx);
    } catch(SQLException sqlEx) {
      System.out.println("SQL Exception :" + sqlEx);
    }
  }

  public String[] findCustomersWhoOrderedForMoreThan(float value)
  {

    // Get customer query info
    Preferences preferences =
      Preferences.userNodeForPackage(
        ejava.corba.NamingContextUtility.class);

    // Get customer query
    String customerQueryString = preferences.get("CUSTOMER_QUERY",
                                                 null);

    if(customerQueryString == null) {
      System.out.println("CUSTOMER_QUERY in  file is null");
    }

    try {

      // Get results from query
      ResultSet rs = getResultSet(customerQueryString,
                                  new Float(value));
      ResultSetMetaData rsMetaData  = rs.getMetaData();
      int               nColumns    = rsMetaData.getColumnCount();
      Vector            customerIDs = new Vector();

      // Create and bind CustomerImplementation per result
      //    and also set IDs into customerIDs Vector
      while(rs.next()) {
        String                 customerID = rs.getString("CUSTOMER_ID");
        String                 firstName   = rs.getString("FIRST_NAME");
        String                 lastName    = rs.getString("LAST_NAME");
        String                 address1    = rs.getString("ADDRESS_1");
        String                 address2    = rs.getString("ADDRESS_2");
        String                 city        = rs.getString("CITY");
        String                 state       = rs.getString("STATE");
        String                 phoneNumber = rs.getString("PHONE");
        String                 eMail       = rs.getString("EMAIL");
        String                 zip         = rs.getString("ZIP_CODE");
        CustomerImplementation custImpl = new CustomerImplementation();

        custImpl.firstName(firstName);
        custImpl.lastName(lastName);
        custImpl.address1(address1);
        custImpl.address2(address2);
        custImpl.city(city);
        custImpl.state(state);
        custImpl.zip(zip);
        custImpl.emailAddress(eMail);
        custImpl.phoneNumber(phoneNumber);

        String name = customerID;

        custImpl.bind(name);
        customerIDs.addElement(customerID);
      }

      // Create and return String array of Customer IDs
      String[] registeredNames = new String[customerIDs.size()];

      for(int i = 0; i < registeredNames.length; i++) {
        registeredNames[i] = (String) customerIDs.elementAt(i);
      }

      return registeredNames;
    } catch(SQLException sqlEx) {
      System.out.println(sqlEx);
      sqlEx.printStackTrace();
    }

    return new String[0];
  }

  public void findOrdersWhichAreFrom(String state)
  {

    // Get order query information from properties data
    Preferences preferences =
      Preferences.userNodeForPackage(
        ejava.corba.NamingContextUtility.class);
    String orderQueryString = preferences.get("ORDER_QUERY", null);

    if(orderQueryString == null) {
      System.out.println("ORDER_QUERY in  file is null");
    }

    try {

      // Get results from query
      ResultSet         rs = this.getResultSet(orderQueryString, state);
      ResultSetMetaData rsMetaData = rs.getMetaData();
      int               nColumns   = rsMetaData.getColumnCount();
      Vector            orders     = new Vector();

      // Create an Order for each result and add to Vector
      while(rs.next()) {
        int    orderID    = rs.getInt("ORDER_ID");
        Date   orderDate  = rs.getDate("ORDER_DATE");
        double orderValue = rs.getDouble("TOTAL_PRICE");
        Order order = new Order(orderID, orderDate.toString(),
                                orderValue);

        orders.addElement(order);
      }

      // Create array of orders
      Order[] returnOrders = new Order[orders.size()];

      for(int i = 0; i < orders.size(); i++) {
        returnOrders[i] = (Order) orders.elementAt(i);
      }

      // Add array of orders to client callback
      if(this.clientReceiver != null) {
        System.out.println("Sending Orders :" + returnOrders);
        clientReceiver.setResultOrders(returnOrders);
      }
    } catch(SQLException sqlEx) {
      System.out.println(sqlEx);
      sqlEx.printStackTrace();
    }
  }

  public void register(String name, ClientReceiver clientReceiver)
  {

    System.out.println("registered Client :" + clientReceiver);

    this.clientReceiver     = clientReceiver;
    this.clientReceiverName = name;
  }

  private ResultSet getResultSet(String sqlQuery,
                                 java.lang.Object whereValue)
                                     throws SQLException
  {

    PreparedStatement preparedStatement =
      connection.prepareStatement(sqlQuery);

    preparedStatement.setObject(1, whereValue);

    ResultSet rs = preparedStatement.executeQuery();

    return rs;
  }
}
