package ejava.ejb.entity.credit;

import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import java.util.Hashtable;

import java.sql.Connection;
import java.sql.SQLException;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.ejb.EJBException;
import java.util.Vector;
import java.util.Enumeration;
import javax.ejb.FinderException;
import ejava.ejb.EJBHelper;
import java.util.Collection;
import java.util.ArrayList;

public class CreditEJBean implements EntityBean {
  private EntityContext entityContext;
  private Hashtable creditInformation; // Stores credit information

  /**
   * Create CREDIT table entry based on iniital creditInformation
   */
  public CreditPrimaryKey ejbCreate(Hashtable creditInformation)
    throws CreateException {
    CreditDBAccess dbAccess = new CreditDBAccess();
    Connection connection = null;
    try {
      // Get connection, set on CreditDBAccess, and perform INSERT
      connection = EJBHelper.getDBConnection();
      dbAccess.setConnection(connection);
      dbAccess.insert(creditInformation);
      this.creditInformation = creditInformation;

    } catch (java.sql.SQLException sqlException) {
      sqlException.printStackTrace();
      throw new CreateException(
        "SQL Exception in create:" + sqlException);
    } catch (NamingException namingException) {
      namingException.printStackTrace();
      throw new CreateException(namingException.getMessage());
    } finally {
      try {
        if (connection != null) {
          connection.close();
        }
      } catch (SQLException sqlException) {
        throw new CreateException(
          "SQL Exception in create:" + sqlException);
      }
    }
    // Create primary key and return
    CreditPrimaryKey primaryKey = new CreditPrimaryKey();
    primaryKey.customerID =
      ((Integer) creditInformation.get(CreditDefinition.CUSTOMER_ID))
        .intValue();
    primaryKey.orderID =
      ((Integer) creditInformation.get(CreditDefinition.ORDER_ID))
        .intValue();
    primaryKey.cardNumber =
      ((String) creditInformation.get(CreditDefinition.CARD_NUMBER));
    return primaryKey;
  }

  /**
   * Empty default implementations
   */
  public void ejbPostCreate(Hashtable creditInformation)
    throws CreateException
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  /** Verify that primary key for Credit exists */
  public CreditPrimaryKey ejbFindByPrimaryKey(CreditPrimaryKey primaryKey)
    throws FinderException {
    if ((primaryKey == null)) {
      throw new FinderException("PrimaryKey should not be null");
    }

    // Throws exception if cannot load based on this key
    loadCreditData(primaryKey);

    return primaryKey;
  }

  /**
   * Look up all primary keys for Credit objects with customer ID
   */
  public Collection ejbFindByCustomerID(int customerID)
    throws FinderException {
    Connection connection = null;
    CreditDBAccess dbAccess = new CreditDBAccess();
    try {
      // Get/set connection, SELECT credit info from database
      connection = EJBHelper.getDBConnection();
      dbAccess.setConnection(connection);
      Vector results = dbAccess.select(customerID);
      // Return  results Vector
      return results;
    } catch (SQLException sqlException) {
      sqlException.printStackTrace();
      throw new FinderException(sqlException.getMessage());
    } catch (NamingException namingException) {
      throw new FinderException(namingException.getMessage());
    } finally {
      try {
        if (connection != null) {
          connection.close();
        }
      } catch (Exception e) {
        throw new EJBException(e.getMessage());
      }
    }
  }

  /**
   * Look up primary keys for Credit with customer ID and order ID
   */
  public Collection ejbFindByCustomerIDAndOrderID(
    int customerID,
    int orderID)
    throws FinderException {

    Connection connection = null;
    CreditDBAccess dbAccess = new CreditDBAccess();
    try {
      // Get/Set connection, SELECT credit information
      connection = EJBHelper.getDBConnection();
      dbAccess.setConnection(connection);
      Vector results = dbAccess.select(customerID, orderID);
      // Return  Credit info Hashtables
      return results;
    } catch (SQLException sqlException) {
      sqlException.printStackTrace();
      throw new FinderException(sqlException.getMessage());
    } catch (NamingException namingException) {
      namingException.printStackTrace();
      throw new FinderException(namingException.getMessage());
    } finally {
      try {
        if (connection != null) {
          connection.close();
        }
      } catch (Exception exception) {
        throw new EJBException(exception.getMessage());
      }
    }
  }

  /**
   * Remove data source entry associated with this bean from DBMS
   */
  public void ejbRemove() {
    Connection connection = null;
    CreditDBAccess dbAccess = new CreditDBAccess();
    try {
      // Get/Set Connection, DELETE this credit info from DB
      connection = EJBHelper.getDBConnection();
      dbAccess.setConnection(connection);
      dbAccess.delete((CreditPrimaryKey) entityContext.getPrimaryKey());
    } catch (SQLException sqlException) {
      sqlException.printStackTrace();
    } catch (NamingException namingException) {
      namingException.printStackTrace();
    } finally {
      try {
        if (connection != null) {
          connection.close();
        }
      } catch (Exception exception) {
        exception.printStackTrace();
      }
    }
  }

  /**
   * Store EJB data to database
   */
  public void ejbStore() {
    Connection connection = null;
    CreditDBAccess dbAccess = new CreditDBAccess();
    try {
      // Get/Set connection, UPDATE credit info in DB
      connection = EJBHelper.getDBConnection();
      dbAccess.setConnection(connection);
      dbAccess.update(creditInformation);
    } catch (SQLException sqlException) {
      sqlException.printStackTrace();
    } catch (NamingException namingException) {
      namingException.printStackTrace();
    } finally {
      try {
        if (connection != null) {
          connection.close();
        }
      } catch (Exception exception) {
        exception.printStackTrace();
      }
    }
  }

  /**
   * Load EJB data from database
   */
  public void ejbLoad() {

    // Get primary key
    CreditPrimaryKey primaryKey =
      (CreditPrimaryKey) entityContext.getPrimaryKey();
    System.out.println("ejbLoad called :");
    // Now load the actual data based on primary key
    loadCreditData(primaryKey);
    System.out.println(primaryKey);
  }

  /**
   * Private method to load credit data into bean using primary key
   */
  private void loadCreditData(CreditPrimaryKey primaryKey) {
    Connection connection = null;
    CreditDBAccess dbAccess = new CreditDBAccess();

    try {
      // Get Connection, set connection, SELECT credit info
      connection = EJBHelper.getDBConnection();
      dbAccess.setConnection(connection);
      creditInformation =
        dbAccess.select(
          primaryKey.customerID,
          primaryKey.orderID,
          primaryKey.cardNumber);
    } catch (SQLException sqlException) {
      sqlException.printStackTrace();
      throw new EJBException(sqlException.getMessage());
    } catch (NamingException namingException) {
      namingException.printStackTrace();
      throw new EJBException(namingException.getMessage());
    } finally {
      try {
        if (connection != null) {
          connection.close();
        }
      } catch (Exception exception) {
        throw new EJBException(exception.getMessage());
      }
    }
  }

  /**
   * Sets the EntityContext for the EJBean.
   */
  public void setEntityContext(EntityContext entityContext) {
    this.entityContext = entityContext;
  }

  /**
   * Unsets the EntityContext for the EJBean.
   */
  public void unsetEntityContext() {
    entityContext = null;
  }

  /**
   * Set credit information onto bean
   */
  public void setCreditInformation(Hashtable creditInformation) {
    this.creditInformation = creditInformation;
  }

  /**
   * Retrieve credit info from bean.
   */
  public Hashtable getCreditInformation() {
    System.out.println("Sending Before :" + this.creditInformation);
    return creditInformation;
  }
}
