package ejava.ejb.entity.order;

import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import java.util.Hashtable;
import java.sql.Date;
import java.util.Collection;
import java.util.Iterator;
import ejava.ejb.entity.customer.CMRCustomerLocal;
import ejava.ejb.entity.customer.CMRCustomerLocalHome;
import ejava.ejb.entity.order.CMROrderLocal;
import ejava.ejb.entity.item.CMRItemLocal;
import ejava.ejb.entity.address.CMRAddressLocal;
import ejava.ejb.EJBHelper;

/**
  * CMROrderEJBean is a CMP EntityBean implementation.
  */
public abstract class CMROrderEJBean extends OrderEJBean {
  private CMRCustomerLocalHome customerHome = null;

  /**
   * Abstract relationship accessors for one to many
   * relationship between an Order and Item bean.
   */
  public abstract Collection getItems();
  public abstract void setItems(Collection items);

  /**
   * Abstract relationship accessors for many to one
   * relationship between an Order and Customer bean.
   */
  public abstract CMRCustomerLocal getCustomer();
  public abstract void setCustomer(CMRCustomerLocal customer);

  /**
   * Abstract relationship accessors for one to many
   * relationship between an Order and Address bean.
   */
  public abstract Collection getAddresses();
  public abstract void setAddresses(Collection addresses);

  /**
   * Abstract ejbSelectXXX() helper method for selecting
   * a collection of items in an order.
   */
  public abstract Collection ejbSelectItemsInAnOrder(Integer orderID)
    throws FinderException;

  /**
   * Abstract ejbSelectXXX() helper method for selecting
   * a collection of orders for a customer.
   */
  public abstract Collection ejbSelectOrdersOfACustomer(Integer customerID)
    throws FinderException;

  /**
   * An ejbHomeXXX() home business method for returning the number
   * of items associated with an order.
   */
  public int ejbHomeNumberOfItemsInAnOrder(Integer orderID)
    throws FinderException {
    return ejbSelectItemsInAnOrder(orderID).size();
  }

  /**
   * An ejbHomeXXX() home business method for returning the average
   * order value of a particular customer.
   */
  public float ejbHomeAverageOrderValue(Integer customerID)
    throws FinderException {
    System.out.println("Inside ejbHomeAverageOrderValue");
    int nItems = 0;
    float totalValue = 0.0f;

    // Iterate through collection of orders for a customer
    Collection orders = ejbSelectOrdersOfACustomer(customerID);
    Iterator ordersIterator = orders.iterator();
    while (ordersIterator.hasNext()) {
      // Iterate through collection of items per order
      Collection items =
        ((CMROrderLocal) ordersIterator.next()).getItems();
      Iterator iterator = items.iterator();
      while (iterator.hasNext()) {
        // Accumulate item price and increment item count
        CMRItemLocal item = (CMRItemLocal) iterator.next();
        nItems++;
        totalValue += item.getTotalPrice();
        System.out.println(
          "ejbHomeAverageOrderValue: currentTotal:" + totalValue);
      }
    }
    // Return average price of items for the customer
    float returnValue = 0.0f;
    if (nItems > 0)
      returnValue = totalValue / nItems;
    return returnValue;
  }

  /**
   * Standard ejbCreate() method for setting initial state.
   * Does not requires special linking ID for customer relationship
   * since relationship is implied by the CMP relationship field.
   */
  public Integer ejbCreate(Integer orderID, Date orderDate,
                           String shippingInstructions,
                           Date shipDate, double shipWeight,
                           double shipCharge, Date paidDate)
      throws CreateException
  {
    this.setOrderID(orderID);
    this.setOrderDate(orderDate);
    this.setShippingInstructions(shippingInstructions);
    this.setShipDate(shipDate);
    this.setShipDate(shipDate);
    this.setShipWeight(shipWeight);
    this.setShipCharge(shipCharge);
    this.setPaidDate(paidDate);
    return null;
  }

  /**
   * Corresponding ejbPostCreate() for ejbCreate() method.
   */
  public void ejbPostCreate(Integer orderID, Date orderDate,
                            String shippingInstructions,
                            Date shipDate, double shipWeight,
                            double shipCharge, Date paidDate)
      throws CreateException
  {
  }
}
