package ejava.ejb.entity.order;

import javax.ejb.EJBLocalHome;
import javax.ejb.CreateException;
import javax.ejb.FinderException;
import java.sql.Date;
import java.util.Collection;

/**
  * This is home interface for EntityBean Order.
  */
public interface CMROrderLocalHome extends EJBLocalHome {

  /**
   * Standard create() method for setting initial state.
   * Does not requires special linking ID for customer relationship
   * since relationship is implied by the CMP relationship field.
   */
  public CMROrderLocal create(Integer orderID, Date orderDate,
                              String shippingInstructions,
                              Date shipDate, double shipWeight,
                              double shipCharge, Date paidDate)
    throws CreateException;

  /**
   * Given an order ID primary key, this method will
   * return an CMROrderLocal object reference if the bean
   * can be found.
   */
  public CMROrderLocal findByPrimaryKey(Integer orderID)
    throws FinderException;

  /**
   * Given a customer ID this method will return a collection
   * of CMROrderLocal object references if any such beans
   * can be found.
   */
  public Collection findByCustomerID(Integer customerID)
    throws FinderException;

  /**
   * Find a collection of CMROrderLocal object references
   * that fall within a particular order ranger value.
   */
  public Collection findAllOrdersInRange(double startValue,
                                         double endVlaue)
    throws FinderException;

  /**
   * A home business method for returning the number
   * of items associated with an order.
   */
  public int numberOfItemsInAnOrder(Integer orderID)
    throws FinderException;

  /**
   * A home business method for returning the average
   * order value of a particular customer.
   */
  public float averageOrderValue(Integer customerID)
    throws FinderException;
}
