package ejava.ejb.security.jaas;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.callback.TextOutputCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.TextInputCallback;
import javax.security.auth.callback.NameCallback;
import weblogic.security.auth.callback.URLCallback;

class TestCallbackHandler implements CallbackHandler {
	private String username = null;
	private String password = null;

	public TestCallbackHandler() {
	}

	/**
	 * Implement CallbackHandler interface
	 */
	public void handle(Callback[] callbacks)
		throws IOException, UnsupportedCallbackException {

    // For each Callback passed to handler, test type and process it
		for (int i = 0; i < callbacks.length; ++i) {
			if (callbacks[i] instanceof TextOutputCallback) {
        useTextOutputCallback((TextOutputCallback) callbacks[i]);
			} else if (callbacks[i] instanceof NameCallback) {
        useNameCallback((NameCallback) callbacks[i]);
			} else if (callbacks[i] instanceof PasswordCallback) {
        usePasswordCallback((PasswordCallback) callbacks[i]);
			} else if (callbacks[i] instanceof URLCallback) {
        useWebLogicURLCallback((URLCallback) callbacks[i]);
			} else {
				throw new UnsupportedCallbackException(callbacks[i],
                                              "Unrecognized Callback");
			}
		}
	}

  private void useTextOutputCallback(TextOutputCallback callback){
    // Display the message
    System.out.println(callback.getMessageType() + ": "
                      +callback.getMessage());
  }

  private void useNameCallback(NameCallback callback) throws IOException{
    // If username not supplied on cmd line, read
    // the user name from command prompt
    System.out.print(callback.getPrompt());
    System.out.flush();
    callback.setName((new BufferedReader
                      (new InputStreamReader(System.in))).readLine());
  }

  private void usePasswordCallback(PasswordCallback callback) throws IOException{
    // If password not supplied on cmd line,
    // read  password from commond prompt.
    System.out.print(callback.getPrompt());
    System.out.flush();
    String pwd = (new BufferedReader
                 (new InputStreamReader(System.in))).readLine();
    char[] passwordArray = new char[pwd.length()];
    for (int passIdx = 0; passIdx < pwd.length(); passIdx++)
      passwordArray[passIdx] = pwd.charAt(passIdx);
    callback.setPassword(passwordArray);
  }

  private void useWebLogicURLCallback(URLCallback callback){
    // read url value from Java VM environment varaibles
    callback.setURL(System.getProperty("java.naming.provider.url"));
  }
}
