package ejava.ejb.session.browse;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import java.rmi.RemoteException;
import javax.rmi.PortableRemoteObject;
import java.util.Collection;
import java.util.Iterator;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Context;
import ejava.ejb.EJBHelper;

import ejava.ejb.entity.tshirt.TShirtLocal;
import ejava.ejb.entity.tshirt.TShirtLocalHome;
import ejava.servlets.TShirt;
import java.util.Vector;

public class BrowseEJBean implements SessionBean {
   // EJB session context
   private SessionContext sessionContext;
   // Home interface to a TShirt entity bean
   private TShirtLocalHome tshirtHome;

   /**
    *  Standard session EJB interfaces implemented
    */
   public void setSessionContext(SessionContext aCtx)
      throws RemoteException {
      sessionContext = aCtx;
      try {
         lookupTShirtHome();
      } catch (NamingException ne) {
         throw new RemoteException(ne.getMessage());
      }
   }

   public void ejbCreate() { /* no creation logic */
   }

   public void ejbRemove() { /* no removal logic */
   }

   public void ejbPassivate() { /* no state persistence */
   }

   public void ejbActivate() { /* no state activation */
   }

   /**
    * Finds the TShirt Home Object
    */
   private void lookupTShirtHome() throws NamingException {
      if (tshirtHome == null) {
         InitialContext context = new InitialContext();
         tshirtHome =
            (TShirtLocalHome)context.lookup("java:comp/env/ejb/tshirtlocal");
         context.close();
      }
   }

   /**
    * Find TShirt entity beans and create Serialized versions
    */
   public Vector queryShirts(String size, String color)
   {
      System.out.println("Querying 123 :");
      // Create collections for entity beans and serialized objects
      Collection queriedShirts = null;
      Vector returnedShirts = new Vector();

      try {
         System.out.println("Querying :1");
         // Find a collection of TShirt entity beans in the database
         queriedShirts = tshirtHome.findBySizeAndColor(size, color);
         System.out.println("Querying :2");
         Iterator iterator = queriedShirts.iterator();
         // While cycling through TShirt entity object collection...
         while (iterator.hasNext()) {
            System.out.println("Querying :3");
            // Get TShirt entity bean
            ejava.ejb.entity.tshirt.TShirtLocal beanShirt =
               (ejava.ejb.entity.tshirt.TShirtLocal) iterator.next();

            // Create Serialized TShirt object from entity bean data
            TShirt clientShirt =
               new TShirt(
                  beanShirt.getShirtID().intValue(),
                  beanShirt.getShirtSize(),
                  beanShirt.getColor(),
                  beanShirt.getDesignFront(),
                  beanShirt.getDesignBack(),
                  beanShirt.getPictureFront(),
                  beanShirt.getPictureBack(),
                  beanShirt.getUnitPrice());

            // Add the TShirt object to a return vector
            returnedShirts.addElement(clientShirt);
         }
      } catch (FinderException finderException) {
         finderException.printStackTrace();
      }

      // Return the Vector of Serialized TShirt objects
      return returnedShirts;
   }
}

