package ejava.ejb.session.creditmgr;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;

public class CreditManagerEJBean implements SessionBean
{
  private SessionContext ctx;

  // Standard interfaces

  public void setSessionContext(SessionContext aCtx){ ctx = aCtx; }

  /**
   *  Empty default implementations of SessionBean interface
   */
  public void ejbCreate()
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbPassivate()
  {
  }

  public void ejbActivate()
  {
  }

  // Application-specific interfaces

  /**
   * Hand off info to credit management handler.
   */
  public void createCreditRecord(int customerID,
                                 String cardType,
                                 String cardName,
                                 String cardNumber,
                                 java.util.Date expirationDate,
                                 String defaultConfirmationNumber)
  {
    // If configured to use handler, then use it
    if(shouldUseHandler()){
      ejava.ejb.session.creditmgr.CreditManagerHandler.handleNewCredit(
                        customerID, cardType, cardName, cardNumber,
                        expirationDate, defaultConfirmationNumber);
    }
    else{ // Otherwise, use default handler from this chapter
      this.handleNewCredit(customerID, cardType, cardName, cardNumber,
                           expirationDate, defaultConfirmationNumber);
    }
  }

  /**
   * Determine if we should use handler or default
   */
  private boolean shouldUseHandler()
  {
    // Local variable to store credit management handler flag
    boolean hasHandler = false;

    // Get initial context and determine if need use special handler
    try{
      InitialContext ctx = new InitialContext();
      Boolean handler = (Boolean) ctx.lookup("java:comp/env/Handler");
      hasHandler = handler.booleanValue();
    }
    catch(Exception namingException){
      System.out.println("No special handler defined. Using default.");
    }

    return hasHandler;
  }

  /**
   * Default print out of credit data
   */
  private void handleNewCredit(int customerID,
                              String cardType,
                              String cardName,
                              String cardNumber,
                              java.util.Date expirationDate,
                              String defaultConfirmationNumber)
  {
    // Print out all credit information as the default functionality
    System.out.println("\nCREATE: Credit Information Record");
    System.out.println("  Customer ID: " + customerID);
    System.out.println(  "  Card Type: " + cardType
                       + "    Card Name: " + cardName);
    System.out.println("  Card Number :" +cardNumber);
    System.out.println(  "  Expire Date: " + expirationDate.toString()
                    + " Confirm Number: " + defaultConfirmationNumber);
  }
}
