package ejava.ejb.session.customermgr;

import javax.ejb.EJBLocalObject;
import java.util.Vector;
import ejava.servlets.Customer;


/**
 * The methods in this interface are the one client calls to
 * query for Orders.
 */
public interface CustomerManagerLocal extends EJBLocalObject
{
   /**
    * Query All Customers
    */
   public Vector getAllCustomers();

   /**
    * Get a Customer detail Information
    */
   public Customer getCustomerDetail(int customerID);

   /**
    * Get Customers whose orders are in the given range
    */
   public Vector getCustomersWithOrdersInRange(double minValue,
                                               double maxValue);

   /**
    * Get Customers who bought a certain size shirts.
    */
   public Vector getCustomersWhoBought(String shirtsSize);

   /**
    * Get Customers who did not buy a certain size shirts.
    */
   public Vector getCustomersWhoDidNotBuy(String shirtsSize);

   /**
    * Get Customers by their first name
    */
   public Vector getCustomersWhoseFirstNameLike();

   /**
    * Get Customers who did not have any Orders
    */
   public Vector getCustomersWhoDoNotHaveOrders();

   /**
    * Get Customers who are from any of these states
    */
   public Vector getCustomersWhoAreFromCapitalArea();

   /**
    * Create New Customer
    */
   public Integer createNewCustomer(Customer customer);

   /**
    * performs authentication on user
    */
   public Customer login(String email, String password)
     throws FailedToLoginException;

   /**
    * registers a new Customer
    */
   public Integer register(Customer customer)
     throws  FailedToRegisterException;

   /**
    * sends the user a password reminder email
    */
   public void remindPassword(String email);

}


