package ejava.ejb.session.ordermgr;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.CreateException;
import javax.ejb.EJBException;
import java.rmi.RemoteException;
import javax.ejb.FinderException;
import javax.rmi.PortableRemoteObject;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Context;
import ejava.ejb.EJBHelper;

import ejava.ejb.entity.tshirt.CMRTShirtLocal;
import ejava.ejb.entity.item.CMRItemLocal;
import ejava.ejb.entity.item.CMRItemLocalHome;
import ejava.ejb.entity.order.CMROrderLocal;
import ejava.ejb.entity.order.CMROrderLocalHome;
import ejava.ejb.entity.customer.CMRCustomerLocalHome;
import ejava.ejb.entity.customer.CMRCustomerLocal;
import ejava.ejb.entity.address.CMRAddressLocal;
import ejava.ejb.entity.address.CMRAddressLocalHome;
import ejava.ejb.entity.item.CMRItemLocal;
import ejava.servlets.TShirt;
import ejava.servlets.Item;
import ejava.servlets.Order;
import ejava.servlets.Customer;
import ejava.servlets.Address;

public class OrderManagerEJBean implements SessionBean {
  // Order Manager bean session attributes
  private SessionContext sessionContext;
  private CMROrderLocalHome orderHome;
  private CMRCustomerLocalHome customerHome;
  private CMRAddressLocalHome addressHome;
  private CMRItemLocalHome itemHome;
  private static final String BILLING = "BILLING";
  private static final String SHIPPING = "SHIPPING";

  /**
   *  Default empty implementation of SessionBean interface
   */
  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbCreate() throws CreateException
  {
  }

  /**
   * This method is required by the EJB Specification for
   * implementing a SessionBean to set the context.
   */
  public void setSessionContext(SessionContext sessionContext)
    throws RemoteException {
    this.sessionContext = sessionContext;
    try {
      lookupOrderHome();
      lookupCustomerHome();
      lookupAddressHome();
      lookupItemHome();
    } catch (NamingException ne) {
      throw new RemoteException(ne.getMessage());
    }
  }

  /**
   * Return a SessionContext handle
   */
  protected SessionContext getSessionContext(){
    return sessionContext;
  }

  /**
   * Query for the orders in the DBMS and return them.
   */
  public Vector queryOrders(int customerID) {
    // Create order collections
    Collection queriedOrders = null;
    Vector returnedOrders = new Vector();
    try {
      System.out.println("Querying Orders :");
      queriedOrders =
        orderHome.findByCustomerID(new Integer(customerID));
      // Get serialized form of orders for sending to client
      returnedOrders = getOrdersToSendToClient(queriedOrders);
    } catch (FinderException fe) {
      fe.printStackTrace();
      throw new EJBException(fe.getMessage());
    }

    // Send serialized order collection to client
    return returnedOrders;
  }

  /**
   * Get a customer object associated with the order
   */
  public Customer getCustomer(int orderID) throws FinderException {
    CMROrderLocal orderLocal =
         this.orderHome.findByPrimaryKey(new Integer(orderID));
    Integer customerID = orderLocal.getCustomerID();
    CMRCustomerLocal customerLocal =
         customerHome.findByPrimaryKey(customerID);
    System.out.println("The order :"+orderLocal.getOrderID()+
            ": belong to :"+ "Customer ID :"+customerLocal.getCustomerID());
    Customer customer = fillReturnCustomer(customerLocal);

    return customer;
  }

  /**
   * Create a Customer object based on local customer interface
   */
  private Customer fillReturnCustomer(CMRCustomerLocal customerLocal) {
    Address address =
      new Address(
        customerLocal.getAddress_1(),
        customerLocal.getAddress_2(),
        customerLocal.getCity(),
        customerLocal.getState(),
        customerLocal.getZipCode());
    Customer customer =
      new Customer(
        customerLocal.getFirstName(),
        customerLocal.getLastName(),
        customerLocal.getMiddleName(),
        address,
        customerLocal.getPhone(),
        customerLocal.getEmail(),
        null);
    return customer;

  }

  /**
   * Get serializeable Orders to send to client
   */
  private Vector getOrdersToSendToClient(Collection orders) {
    Vector returnedOrders = new Vector();

    Iterator iterator = orders.iterator();

    // While have orders in the collection...
    while (iterator.hasNext()) {
      // Extract an order entity bean
      ejava.ejb.entity.order.CMROrderLocal beanOrder =
        (ejava.ejb.entity.order.CMROrderLocal) iterator.next();

      // Create and initialize serializable form of Order with
      // entity bean order info
      Order clientOrder = new Order();
      clientOrder.setOrderID(beanOrder.getOrderID().intValue());
      clientOrder.setPaidDate(beanOrder.getPaidDate());
      clientOrder.setShipDate(beanOrder.getShipDate());
      clientOrder.setShipWeight(beanOrder.getShipWeight());
      clientOrder.setOrderDate(beanOrder.getOrderDate());
      clientOrder.setShippingInstructions(
        beanOrder.getShippingInstructions());
      // Get array of serializable TShirt objects
      // associated with order
      TShirt[] items = getItemsForTheOrder(beanOrder);
      // Set the serializable TShirt array onto Serializable order
      clientOrder.setItems(items);
      // Set Address Information
      setOrderAddress(beanOrder, clientOrder);
      // Add serializable order to return vector
      returnedOrders.addElement(clientOrder);
    }
    // Return vector of serializable orders
    return returnedOrders;
  }

  /**
   * Read info from an local order entity bean and
   * sent into a serialized order object
   */
  private void setOrderAddress(CMROrderLocal orderLocal,
                               Order clientOrder)
  {
    Collection addresses = orderLocal.getAddresses();
    Iterator it = addresses.iterator();
    while (it.hasNext()) {
      CMRAddressLocal addressLocal = (CMRAddressLocal) it.next();
      if (addressLocal.getAddressType().equalsIgnoreCase(BILLING)) {
        Address billingAddress =
          new Address(
            addressLocal.getAddress_1(),
            addressLocal.getAddress_2(),
            addressLocal.getCity(),
            addressLocal.getState(),
            addressLocal.getZipCode());
        clientOrder.setBillingAddress(billingAddress);
      } else {
        Address shippingAddress =
          new Address(
            addressLocal.getAddress_1(),
            addressLocal.getAddress_2(),
            addressLocal.getCity(),
            addressLocal.getState(),
            addressLocal.getZipCode());
        clientOrder.setShippingAddress(shippingAddress);
      }
    }
  }

  /**
   * Get serialized array of Tshirt objects from an order
   */
  private TShirt[] getItemsForTheOrder(CMROrderLocal orderLocal) {
    // Construct a Vector
    Vector itemsForTheOrder = new Vector();
    // Find Item beans associated with the order
    Collection items = orderLocal.getItems();
    Iterator iterator = items.iterator();
    // For each returned Item...
    while (iterator.hasNext()) {
      // Get an Item
      ejava.ejb.entity.item.CMRItemLocal beanItem =
        (ejava.ejb.entity.item.CMRItemLocal) iterator.next();
      // Get a TShirt entity bean
      Collection beanShirts = beanItem.getTShirts();
      Iterator beanShirtsIterator = beanShirts.iterator();
      while (beanShirtsIterator.hasNext()) {
        CMRTShirtLocal beanShirt =
          (CMRTShirtLocal) beanShirtsIterator.next();
        // Create a Serializable TShirt with initialized data
        TShirt clientTShirt =
          new TShirt(
            beanShirt.getShirtID().intValue(),
            beanShirt.getShirtSize(),
            beanShirt.getColor(),
            beanShirt.getDesignBack(),
            beanShirt.getDesignBack(),
            beanShirt.getPictureFront(),
            beanShirt.getPictureBack(),
            beanShirt.getUnitPrice());
        clientTShirt.setItemID(beanItem.getItemID().intValue());
        clientTShirt.setQuantity(beanItem.getQuantity().intValue());
        clientTShirt.setTotalPrice(beanItem.getTotalPrice());
        // Add Serializable TShirt info into Vector
        itemsForTheOrder.addElement(clientTShirt);
      }
    }

    // Make array of Tshirts
    TShirt[] returnItems = new TShirt[itemsForTheOrder.size()];

    // For each TShirt in the Vector, extract info into array
    for (int i = 0; i < itemsForTheOrder.size(); i++) {
      returnItems[i] = (TShirt) itemsForTheOrder.elementAt(i);
    }

    // Return array of Serializable TShirt objects
    return returnItems;
  }

  /**
   * Find number of items in an order
   */
  public int numberOfItemsInAnOrder(int orderID)
    throws FinderException {
    return this.orderHome.numberOfItemsInAnOrder(new Integer(orderID));
  }

  /**
   * Find average value of an order for a customer
   */
  public float averageOrderValueOfACustomer(int customerID)
    throws FinderException {

    return this.orderHome.averageOrderValue(new Integer(customerID));
  }

  /**
   * Create a new order and return the order ID
   */
  public Integer createNewOrder(Order order) {
    Integer newOrderID = null;
    Integer customerID =
      new Integer(order.getCustomer().getCustomerID());
    try {
      newOrderID = EJBHelper.getNextOrderID();
      CMROrderLocal orderLocal =
        this.orderHome.create(
          newOrderID,
          new java.sql.Date(order.getOrderDate().getTime()),
          order.getShippingInstructions(),
          new java.sql.Date(order.getShipDate().getTime()),
          order.getShipWeight(),
          order.getShipCharge(),
          new java.sql.Date(order.getPaidDate().getTime()));
      System.out.println("Created Order :" + newOrderID);
      CMRCustomerLocal customerLocal =
        this.customerHome.findByPrimaryKey(customerID);
      System.out.println("Found Customer:");
      orderLocal.setCustomer(customerLocal);
      System.out.println("Setting Address :");
      // Set Addresses for the Order
      this.setAdresses(orderLocal, order);
      System.out.println("Setting Items :");
      // Set Items for the order
      this.setItems(orderLocal, order);
    } catch (CreateException ce) {
      throw new EJBException(ce.getMessage());
    } catch (NamingException ne) {
      throw new EJBException(ne.getMessage());
    } catch (SQLException se) {
      throw new EJBException(se.getMessage());
    } catch (FinderException fe) {
      throw new EJBException(fe.getMessage());
    }
    return newOrderID;
  }

  /**
   * Set order items into persistent Order
   */
  private void setItems(CMROrderLocal orderLocal, Order order)
    throws CreateException, SQLException, NamingException {
    TShirt[] items = order.getItems();
    ArrayList itemBeans = null;
    Integer newItemID = EJBHelper.getNextItemID();
    for (int i = 0; i < items.length; i++) {
      System.out.println("new Item ID :" + newItemID);
      Item item = (Item) items[i];
      CMRItemLocal itemLocal =
        itemHome.create(
          newItemID,
          new Integer(item.getQuantity()),
          item.getTotalPrice());
      if (itemBeans == null)
        itemBeans = new ArrayList();
      itemBeans.add(itemLocal);
      newItemID = new Integer(newItemID.intValue() + 1);
    }
    System.out.println(" Item - Order Relations");
    // make the relation between Order and Item
    orderLocal.setItems(itemBeans);
  }

  /**
   * Set addresses to persistent Order
   */
  private void setAdresses(CMROrderLocal orderLocal, Order order)
    throws CreateException {
    if (order.getBillingAddress() != null) {
      String addressType = BILLING;
      CMRAddressLocal address =
        addressHome.create(
          orderLocal.getOrderID(),
          addressType,
          order.getBillingAddress().getAddress_1(),
          order.getBillingAddress().getAddress_2(),
          order.getBillingAddress().getCity(),
          order.getBillingAddress().getState(),
          order.getBillingAddress().getZipCode());
      orderLocal.getAddresses().add(address);
    } else if (order.getShippingAddress() != null) {
      String addressType = SHIPPING;
      CMRAddressLocal address =
        addressHome.create(
          orderLocal.getOrderID(),
          addressType,
          order.getShippingAddress().getAddress_1(),
          order.getShippingAddress().getAddress_2(),
          order.getShippingAddress().getCity(),
          order.getShippingAddress().getState(),
          order.getShippingAddress().getZipCode());
      orderLocal.getAddresses().add(address);
    }
  }

  // Methods used to lookup home interfaces
  private void lookupOrderHome() throws NamingException {
    if (orderHome == null) {
      InitialContext context = new InitialContext();
      orderHome =
        (CMROrderLocalHome) context.lookup(EJBHelper.ORDER_REFNAME);
      context.close();
    }
  }

  private void lookupCustomerHome() throws NamingException {
    // Create initial context
    InitialContext initialContext = new InitialContext();
    // Lookup CMRCustomerLocalHome
    customerHome =
      (CMRCustomerLocalHome) initialContext.lookup(
        EJBHelper.CUSTOMER_REFNAME);
    // Close context
    initialContext.close();
  }

  private void lookupAddressHome() throws NamingException {
    // Create initial context
    InitialContext initialContext = new InitialContext();
    // Lookup CMRAddressLocalHome
    addressHome =
      (CMRAddressLocalHome) initialContext.lookup(
        EJBHelper.ADDRESS_REFNAME);
    // Close context
    initialContext.close();
  }

  private void lookupItemHome() throws NamingException {
    // Create initial context
    InitialContext initialContext = new InitialContext();
    // Lookup CMRItemLocalHome
    itemHome =
      (CMRItemLocalHome) initialContext.lookup(EJBHelper.ITEM_REFNAME);
    // Close context
    initialContext.close();
  }

  // Return customer home to sub-classes
  protected CMRCustomerLocalHome getCustomerHome(){
    return this.customerHome;
  }
}
