package ejava.ejb.session.ordermgr;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Date;
import javax.rmi.PortableRemoteObject;

import ejava.ejb.EJBHelper;
import ejava.servlets.TShirt;
import ejava.servlets.Order;
import ejava.servlets.Item;
import ejava.servlets.Customer;
import ejava.servlets.Address;

public class TestClient {
  private static String userName;
  private static String password;
  private static int queryCustomerID = 101;
  private static int queryOrderID = 1001;

  public static void printOrder(Order order) {
    System.out.println("ID :" + order.getOrderID());
    System.out.println("orderDate :" + order.getOrderDate());
    System.out.println(
      "shipInstruct :" + order.getShippingInstructions());
    System.out.println("shipDate :" + order.getShipDate());
    System.out.println("shipWeight :" + order.getShipWeight());
    System.out.println("paidDate :" + order.getPaidDate());
  }

  private static Integer createNewOrder(OrderManager orderManager)
    throws RemoteException {
    Order order = new Order();
    // Order is created today and paid today and shipped today.
    order.setOrderDate(new Date());
    order.setPaidDate(new Date());
    order.setShippingInstructions("Leave at Door");
    order.setShipWeight(10.8);
    order.setShipCharge(32.54);
    order.setShipDate(new Date());
    // The customer info the order is beeing created for
    Customer customer = new Customer();
    customer.setCustomerID("" + queryCustomerID);
    order.setCustomer(customer);
    order.setBillingAddress(createAnAddress());
    order.setShippingAddress(createAnAddress());
    order.setItems(createItems());
    return orderManager.createNewOrder(order);
  }

  private static Address createAnAddress() {
    Address address = new Address();
    address.setAddress_1(" 2.1 EJB Way");
    address.setAddress_2(" 1");
    address.setCity(" J2EE ");
    address.setState("CA");
    address.setZipCode("14000");
    return address;
  }

  private static TShirt[] createItems() {
    TShirt itemOne = new TShirt();
    itemOne.setQuantity(2);
    itemOne.setTotalPrice(21.0);
    itemOne.setSize("XL");
    itemOne.setColor("White");
    itemOne.setUnitPrice(10.5);

    TShirt itemTwo = new TShirt();
    itemTwo.setQuantity(2);
    itemTwo.setTotalPrice(21.0);
    itemTwo.setSize("L");
    itemTwo.setColor("White");
    itemTwo.setUnitPrice(10.5);
    TShirt[] createdItems = { itemOne, itemTwo };
    return createdItems;
  }

  public static void main(String[] args) {
    try {
      // Create initial context
      Context context = new InitialContext();
      System.out.println(
        " looking for :"
          + EJBHelper.ORDER_MANAGER_HOME
          + " context:"
          + context);

      // Lookup order manager home reference
      Object objectRef = context.lookup(EJBHelper.ORDER_MANAGER_HOME);
      OrderManagerHome orderManagerHome =
        (OrderManagerHome) PortableRemoteObject.narrow(
          objectRef,
          ejava.ejb.session.ordermgr.OrderManagerHome.class);

      // Create an order manager handle reference
      OrderManager orderManager = orderManagerHome.create();

      // Create a new order
      Integer newOrderID = createNewOrder(orderManager);
      System.out.println("New Order ID :" + newOrderID);

      // Query for number of items in the order
      int nItems = orderManager.numberOfItemsInAnOrder(queryOrderID);
      System.out.println(" Number of Items in an Order :" + nItems);

      // Query for orders based on customer ID
      Vector orders = orderManager.queryOrders(queryCustomerID);
      System.out.println("Received Orders :" + orders.getClass());
      for (int i = 0; i < orders.size(); i++) {
        Order order = (Order) orders.elementAt(i);
        System.out.println("Received Order : " + i + ": Order Detail");
        printOrder(order);
      }

      // Get the average order value for a customer
      float avgValue =
        orderManager.averageOrderValueOfACustomer(queryCustomerID);
      System.out.println(
        " Average Order Value of a Customer :" + avgValue);

      // Get a customer associated with an order
      Customer customer
        = orderManager.getCustomer(newOrderID.intValue());
      System.out.println(
        " Customer email address :" + customer.getEmail());
    } catch (Exception exception) {
      exception.printStackTrace();
    }
  }
}
