package ejava.ejb.timer;

import java.rmi.RemoteException;
import javax.ejb.CreateException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.TimedObject;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import java.util.Calendar;
import java.util.Iterator;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import ejava.ejb.entity.customer.CustomerLocal;
import ejava.ejb.entity.customer.CustomerLocalHome;
import ejava.ejb.EJBHelper;

public class MessageTimerEJBean
	implements MessageDrivenBean, MessageListener, TimedObject {
	private MessageDrivenContext messageDrivenContext;
	private CustomerLocalHome customerHome;
	private static final int N_DAYS = 30;
	private long defaultDuration = -1;

	/**
	 * This method is required by the EJB Specification,
	 * but is not used here.
	 */
	public void ejbRemove() {
		System.out.println("ejbRemove called from MessageTimerBean Bean");
	}

	/**
	 * This method is required by the EJB Specification,
	 * but is not used here.
	 *
	 */
	public void ejbCreate() {
		System.out.println("ejbCreate called from MessageTimerBean Bean");
	}

	public void ejbTimeout(Timer timer) {
		System.out.println("MessageTimerBean: ejbTimeout  Called! ");
		System.out.println(
			" I will start sending email now!:" + System.currentTimeMillis());
		try {
			// Send email to Every customer.
			sendEmailToEveryCustomer();
			// Reset timer to default Duration
			resetTimerDuration(defaultDuration);
		} catch (FinderException fe) {
			fe.printStackTrace();
		}
	}

	public void resetTimer(long duration) {
		System.out.println("MessageTimerBean: start createTimer ");
		// Cancel Existing Timers
		cancelExistingTimers();
		// creae new timer with new duration
		resetTimerDuration(duration);
	}

	public void resetTimerDuration(long duration) {
		System.out.println("MessageTimerBean: start resetTimer ");
		TimerService timerService = messageDrivenContext.getTimerService();
		System.out.println("Current Timer Service is : " + timerService);
		Timer timer = timerService.createTimer(duration, "Email Timer");
		System.out.println("New Timer is :" + timer);
	}

	private void cancelExistingTimers() {
		TimerService timerService = messageDrivenContext.getTimerService();
		Iterator timers = timerService.getTimers().iterator();
		while (timers.hasNext()) {
			Timer timer = (Timer) timers.next();
			timer.cancel();
		}
	}

	private void sendEmailToEveryCustomer() throws FinderException {
		Iterator iterator = customerHome.findAll().iterator();
		while (iterator.hasNext()) {
			CustomerLocal customer = (CustomerLocal) iterator.next();
			String emailAddress = customer.getEmail();
			System.out.println(
				"Sending Email to :"
					+ customer.getFirstName()
					+ ","
					+ customer.getLastName());
		}
	}

	/** Lookup Customer Home object */
	private void lookupCustomerHome() throws NamingException {
		if (customerHome == null) {
			InitialContext context = new InitialContext();
			customerHome =
				(CustomerLocalHome) context.lookup(EJBHelper.CUSTOMER_REFNAME);
			context.close();
		}
	}

	/**
	 * Sets the MessageDrivenBean context.
	 *
	 * @param ctx  MessageDrivenContext Context for Bean
	 */
	public void setMessageDrivenContext(MessageDrivenContext ctx) {
		System.out.println("setMessageDrivenContext called");
		Calendar time = Calendar.getInstance();
		// add N_DAYS to today
		time.add(Calendar.DATE, N_DAYS);
		defaultDuration = time.getTimeInMillis();
		this.messageDrivenContext = ctx;
		// lookup customer home
		try {
			lookupCustomerHome();
		} catch (NamingException ne) {
			ne.printStackTrace();
		}
	}

  /**
   * Receives a TextMessge and resets the timer duration
   * passed via the message.
   */
	public void onMessage(Message receivedMessage) {
		try {
			if (receivedMessage instanceof TextMessage) {
				TextMessage message = (TextMessage) receivedMessage;
				System.out.println("Received Duration :" + message.getText());
				long duration = Long.parseLong(message.getText());
				resetTimerDuration(duration);
			} else {
				System.out.println(
					"This bean can consume only"
						+ " messages of TextMessages with long values in Payload");
			}
		} catch (JMSException ex) {
			ex.printStackTrace();
		}
	}
}
