package ejava.integration.jca;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 *
 * Package: ejava.integration.jca
 *
 * Type: ConnectorSampleEJBean.java
 *
 */

import javax.resource.ResourceException;
import javax.resource.NotSupportedException;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.Connection;
import javax.resource.cci.InteractionSpec;
import javax.resource.cci.ConnectionSpec;
import javax.resource.cci.Interaction;
import javax.resource.cci.RecordFactory;
import javax.resource.cci.Record;
import javax.resource.cci.IndexedRecord;
import java.util.Iterator;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.ejb.EJBException;
import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.math.BigDecimal;

//== implementation specific.
import com.sun.connector.cciblackbox.CciConnectionSpec;
import com.sun.connector.cciblackbox.CciInteractionSpec;

public class ConnectorSampleEJBean implements SessionBean {
  // EJB session context
   private SessionContext sessionContext;

  // Standard EJB implementation methods
   public void ejbCreate() throws CreateException {
   }
   public void ejbRemove() {
   }
   public void ejbActivate() {
   }
   public void ejbPassivate() {
   }

  /**
   * Set session context
   */
  public void setSessionContext(SessionContext sessionContext) {
      this.sessionContext = sessionContext;
  }

  /**
   * Business specific method illustrating connector usage
   */
   public double getTotalPriceForOrder(Integer orderID) {
      double totalValue = 0.0;

      try {
         // Create JNDI initial context
         InitialContext initialContext = new InitialContext();

         // Look up ConnectionFactory object
         String factoryName = "java:comp/env/eis/ejavaConnector";
         ConnectionFactory connectionFactory
           = (ConnectionFactory) initialContext.lookup(factoryName);

         // Close JNDI context
         initialContext.close();

         // Get connection spec object (vendor-specific)
         ConnectionSpec connectionSpec = new CciConnectionSpec();
         // Could get authenticating version above too as follows...
         // ConnectionSpec connectionSpec
         //  = new CciConnectionSpec(userName, password);

         // Get the connection from the factory with a connection spec
         Connection connection
           = connectionFactory.getConnection(connectionSpec);

         // Get a record factory
         RecordFactory recordFactory =
               connectionFactory.getRecordFactory();

         // Create and set the input record values
         IndexedRecord inputRecord =
           recordFactory.createIndexedRecord("InputRecord");
         inputRecord.add(orderID);

         // Create an interaction spec (vendor-specific)
          CciInteractionSpec interactionSpec = new CciInteractionSpec();

         // Set the function name
         interactionSpec.setFunctionName("GETTOTALPRICEFORORDER");         

         // Set other example JavaBean properties for schema & catalog
         interactionSpec.setSchema("TSHIRTS");
         interactionSpec.setCatalog(null);

         // Create an interaction and execute the function
         Interaction interaction = connection.createInteraction();
         Record outRecord =
               interaction.execute(interactionSpec, inputRecord);

         // Get and traverse the output record result (one for order ID)
         Iterator iterator = ((IndexedRecord) outRecord).iterator();
         while (iterator.hasNext()) {
            Object value = iterator.next();
            totalValue = ((BigDecimal)value).doubleValue();
         }

         // Now close the interaction and EIS connection
         interaction.close();
         connection.close();
      } catch (ResourceException ex) {
         ex.printStackTrace();
      }catch(NamingException ex){
         ex.printStackTrace();
      }
    // Return the total value for the order ID
      return totalValue;
   }
}
