package ejava.integration.web;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletResponse;
import java.util.Vector;
import java.io.FileInputStream;
import java.io.IOException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.http.Cookie;
import javax.rmi.PortableRemoteObject;
import javax.ejb.CreateException;

import ejava.ejb.session.ordermgr.OrderManagerLocal;
import ejava.ejb.session.ordermgr.OrderManagerLocalHome;
import ejava.ejb.session.browse.BrowseLocalHome;
import ejava.ejb.session.browse.BrowseLocal;
import ejava.ejb.session.shoppingcart.ShoppingCartLocalHome;
import ejava.ejb.session.shoppingcart.ShoppingCartLocal;
import ejava.ejb.session.customermgr.CustomerManagerLocal;
import ejava.ejb.session.customermgr.CustomerManagerLocalHome;
import ejava.ejb.session.customermgr.FailedToRegisterException;
import ejava.ejb.session.customermgr.FailedToLoginException;
import ejava.servlets.Customer;
import ejava.servlets.Address;
import ejava.servlets.ShoppingCart;
import ejava.servlets.TShirt;
import ejava.servlets.Order;

public class WebHelper {
	/** Standard BeeShirts.com Cookie Name **/
	public static final String COOKIE_NAME = "BEESHIRTS";
	public static final String POST_METHOD = "POST";
	public static final String GET_METHOD = "GET";
	public static final String HEADER_METHOD = "HEADER";
	public static final String BROWSED_SHIRTS = "BROWSED_SHIRTS";

	public static final String WELCOME_MESSAGE_ENV = "WelcomeMessage";
	public static final String ENV_PREFIX = "java:comp/env/";

	// different page names
	public static final String ABOUT_US_PAGE = "/AboutUs.jsp";
	public static final String CONTACT_INFO_PAGE =
		"/ContactInformation.jsp";
	public static final String DELIVERY_INFO_PAGE = "/Delivery.jsp";
	public static final String TERMS_AND_CONDITIONS_PAGE =
		"/TermsAndConditions.jsp";
	public static final String ORDER_STATUS_PAGE = "/OrderStatus.jsp";
	public static final String CART_PAGE = "/Cart.jsp";
	public static final String BROWSE_SHIRTS_PAGE = "/BrowseShirts.jsp";
	public static final String REGISTRATION_PAGE = "/Registration.jsp";
	public static final String REMIND_PASSWORD_PAGE =
		"/RemindPassword.jsp";
	public static final String VERIFY_LOGIN_PAGE = "/VerifyLogin.jsp";
	public static final String CHECK_OUT_PAGE = "/CheckOut.jsp";
	public static final String SERVER_INFO = "ServerInformation";

	// different EJBs reference names.
	private static String BROWSE_REFNAME = "java:comp/env/ejb/browse";
	private static String CUSTOMERMANAGER_REFNAME =
		"java:comp/env/ejb/customermgr";
	private static String ORDERMANAGER_REFNAME =
		"java:comp/env/ejb/ordermgr";
	private static String SHOPPINGCART_REFNAME =
		"java:comp/env/ejb/shoopingcart";

	private static OrderManagerLocalHome orderManagerHome;
	private static ShoppingCartLocalHome shoppingCartHome;
	private static BrowseLocalHome browseHome;
	private static CustomerManagerLocalHome customerManagerHome;

	private static final int SIX_MONTHS_IN_SECONDS = 60 * 60 * 24 * 183;

	public static OrderManagerLocalHome getOrderManagerHome() {
		if (orderManagerHome == null) {
			try {
				InitialContext ctx = new InitialContext();
				orderManagerHome =
					(OrderManagerLocalHome) ctx.lookup(ORDERMANAGER_REFNAME);
				ctx.close();
				System.out.println(
					"Got Order Manager Home :" + orderManagerHome);
			} catch (NamingException ne) {
				ne.printStackTrace();
			}
		}
		return orderManagerHome;
	}

	public static BrowseLocalHome getBrowseHome() {
		if (browseHome == null) {
			try {
				InitialContext ctx = new InitialContext();
				browseHome = (BrowseLocalHome) ctx.lookup(BROWSE_REFNAME);
				ctx.close();
				System.out.println("Got Browse Manager Home :" + browseHome);
			} catch (NamingException ne) {
				ne.printStackTrace();
			}
		}
		return browseHome;
	}

	public static ShoppingCartLocalHome getShoppingCartHome() {
		if (shoppingCartHome == null) {
			try {
				InitialContext ctx = new InitialContext();
				shoppingCartHome =
					(ShoppingCartLocalHome) ctx.lookup(SHOPPINGCART_REFNAME);
				ctx.close();
				System.out.println("Got Cart Session :" + shoppingCartHome);
			} catch (NamingException ne) {
				ne.printStackTrace();
			}
		}
		return shoppingCartHome;
	}

	public static CustomerManagerLocalHome getCustomerManagerHome()
		throws CreateException {
		if (customerManagerHome == null) {
			try {
				InitialContext ctx = new InitialContext();
				customerManagerHome =
					(CustomerManagerLocalHome) ctx.lookup(
						CUSTOMERMANAGER_REFNAME);
				ctx.close();
				System.out.println(
					"Got Customer Manager :" + customerManagerHome);
			} catch (NamingException ne) {
				ne.printStackTrace();
			}
		}
		return customerManagerHome;
	}

	public static void setACookieIfNoneAvailable(
		HttpSession session,
		HttpServletResponse response,
		String firstName) {
		System.out.println("In cookie call:" + firstName);
		// Get any cookie value named "BEESHIRTS" from chapter 32's
		// JSPHelper static constant COOKIE_NAME
		String cookieValue = (String) session.getAttribute(COOKIE_NAME);
		System.out.println("In cookie call:" + cookieValue);

		// If no cookie, then add user's first name to cookie
		if (cookieValue == null) {
			// create Cookie for this customer
			Cookie cookie = new Cookie(COOKIE_NAME, firstName);
			// Set maximum age of cookie
			cookie.setMaxAge(SIX_MONTHS_IN_SECONDS);
			// Now add the cookie to the response stream
			response.addCookie(cookie);
		}
	}

	public static Customer registerNewCustomer(Customer customer)
		throws CreateException, FailedToRegisterException {
		if (customerManagerHome == null) {
			getCustomerManagerHome();
		}
		Integer newCustomerID =
			customerManagerHome.create().register(customer);
		customer.setCustomerID(newCustomerID.toString());
		return customer;
	}

	public static Object contextLookup(String entry) {
		Object value = null;
		try {
			InitialContext ctx = new InitialContext();
			value = ctx.lookup(ENV_PREFIX + entry);
		} catch (NamingException ne) {
			System.out.println(ne);
		}

		return value;
	}

	public static String getServerInfo(HttpServletRequest request) {
		String serverHost = request.getServerName();
		int serverPort = request.getServerPort();
		String context = request.getContextPath();
		String protocol = "HTTP://";
		String serverInfo =
			protocol + serverHost + ":" + serverPort + context;
		return serverInfo;
	}

	/**
	* Check whether the standard BEESHIRTS Cookie is available
	*/
	public static String getOurCookieValue(Cookie[] cookies) {
		String returnValue = null;
		if (cookies == null)
			return null;

		// Search cookie list for standard BEESHIRTS cookie
		for (int i = 0; i < cookies.length; i++) {
			Cookie cookie = cookies[i];
			// If retrieved cookie name is standard cookie name: "BEESHIRTS"
			if (cookie.getName().equals(COOKIE_NAME)) {
				// Then set return value equal to value of this cookie
				returnValue = cookie.getValue();
				// Now return cookie value from this method
				return returnValue;
			}
		}
		// Will return null cookie value if made it to this point
		return returnValue;
	}

	public static Vector queryShirts(String size, String color)
		throws CreateException{
    // Create a return value and lookup Browse home interface
		Vector returnValue = null;
		if (browseHome == null) {
			getBrowseHome();
		}
    
    // Create the Browse local interface
		System.out.println("Browse Session ");
		BrowseLocal browseLocal = browseHome.create();
    
    // Induce the Browse interface to return a Vector of shirts
		System.out.println("Browse Session Created ");
		returnValue = browseLocal.queryShirts(size, color);
		return returnValue;
	}

	public static void addAnItemToTheCart(
		HttpSession session,
		int shirtID,
		int numberSelected) {
		try {
			// Retrieve shopping cart from session
			ShoppingCartLocal cart =
				(ShoppingCartLocal) session.getAttribute(
					ShoppingCart.SHOPPING_CART_OBJECT);
			// If no cart, then make a new one
			if (cart == null) {
				if (shoppingCartHome == null) {
					getShoppingCartHome();
				}
				cart = shoppingCartHome.create();
				// Add shopping cart to session
				session.setAttribute(ShoppingCart.SHOPPING_CART_OBJECT, cart);
			}
			// add item to the Cart
			cart.addAnItemToTheCart(shirtID, numberSelected);
		} catch (CreateException ce) {
			ce.printStackTrace();
		}

	}

	public static boolean login(
		HttpSession session,
		String emailAddress,
		String password) {
		System.out.println(emailAddress);
		System.out.println(password);
		try {
			if (customerManagerHome == null) {
				getCustomerManagerHome();
			}
			// Verify login
			Customer customer =
				customerManagerHome.create().login(emailAddress, password);
			if (customer == null) {
				return false;
			}
			// set the current Customer into Session
			session.setAttribute(Customer.CUSTOMER_OBJECT, customer);
			System.out.println(" found a customer :" + customer);
			return true;
		} catch (CreateException createException) {
			//createException.printStackTrace();
			return false;
		} catch (FailedToLoginException failedToLogin) {
			//failedToLogin.printStackTrace();
			return false;
		}
	}

	/**
	  * get Orders for the customer and display the orders
	  */
	public static Vector getOrdersForThisCustomer(HttpSession session)
		throws ServletException {
		Vector returnOrders = null;
		try {
			Customer customer =
				(Customer) session.getAttribute(Customer.CUSTOMER_OBJECT);
			System.out.println("Customer :" + customer);
			if (orderManagerHome == null) {
				getOrderManagerHome();
			}
			Integer customerID = new Integer(customer.getCustomerID());
			// Verify login
			returnOrders =
				orderManagerHome.create().queryOrders(customerID.intValue());
			// System.out.println("Got these orders :"+returnOrders);
		} catch (CreateException createException) {
			createException.printStackTrace();
		}
		return returnOrders;
	}

	public static boolean areThereItemsInTheCart(HttpSession session) {
		// Retrieve shopping cart from session
		ShoppingCartLocal cart =
			(ShoppingCartLocal) session.getAttribute(
				ShoppingCart.SHOPPING_CART_OBJECT);
		if (cart == null) {
			return false;
		} else {
			return true;
		}
	}

	public static Vector getItemsInTheCart(HttpSession session) {
		// Retrieve shopping cart from session
		ShoppingCartLocal cart =
			(ShoppingCartLocal) session.getAttribute(
				ShoppingCart.SHOPPING_CART_OBJECT);
		return cart.getSelectedItemsInTheCart();
	}

	public static Order checkOut(HttpSession session, int customerID) {
		Order order = null;
		try {
			// Retrieve shopping cart from session
			ShoppingCartLocal cart =
				(ShoppingCartLocal) session.getAttribute(
					ShoppingCart.SHOPPING_CART_OBJECT);
			//check out
			order = cart.checkOut(customerID);
			session.setAttribute(Order.ORDER_OBJECT, order);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return order;
	}

	public static void remindPassword(String emailAddress) {
		System.out.println(emailAddress);
		try {
			if (customerManagerHome == null) {
				getCustomerManagerHome();
			}
			// Send An email to User
			customerManagerHome.create().remindPassword(emailAddress);
		} catch (CreateException createException) {
			createException.printStackTrace();
		}
	}
}
