package ejava.javamail;
import java.util.Properties;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;
import javax.mail.NoSuchProviderException;
import javax.mail.AuthenticationFailedException;
import javax.mail.MessagingException;
import javax.mail.Message;
import javax.mail.Flags;
import javax.mail.Address;
import javax.mail.Transport;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.InternetAddress;
import javax.mail.search.OrTerm;
import javax.mail.search.FromStringTerm;
import javax.mail.search.ReceivedDateTerm;
import javax.mail.search.SearchTerm;
import javax.mail.search.ComparisonTerm;
import java.util.prefs.Preferences;
import java.util.Date;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedInputStream;
import javax.mail.search.*;

/**
  * This class works as a Mail Client
  * it can send new message, read messages, and search for messages
  */
public class MailClient
{
  /// SMTP host name
  private String  SMTPHost;
  // Message store host name
  private String  MessageStoreHost;
  // Message Store Server port  -1 is for default Port
  private int MessageStorePort  = -1;
  // Email User name
  private String  userName;
  // Email user password
  private String  password;

  // we read the messsges only from inbox.
  private final String  INBOX = "INBOX";
  // read messages from message store server
  private final String  MSG_STORE_MAIL="pop3";
  // read messages from smtp mail.
  private final String  SMTP_MAIL="smtp";
  // mail Header types
  private final String MAIL_HEADER = "X-Mailer";

  // Mail session handle
  private Session mailSession;
  // Mail message store handle
  private Store   mailStore;
  // Mail folder handle
  private Folder  mailFolder;


  public MailClient()
  {
    // Read properties from preferences  object
    Preferences preferences =
      Preferences.userNodeForPackage(MailClient.class);
    SMTPHost = preferences.get("SMTPHOST","");
    MessageStoreHost = preferences.get("STOREHOST","");
    userName = preferences.get("USERNAME","");
    password = preferences.get("PASSWORD","");
    MessageStorePort = preferences.getInt("STOREPORT",-1);
    System.out.println("Host :"+MessageStoreHost+" PORT :"+MessageStorePort);
  }

  public void connectToMailServer()
    throws NoSuchProviderException, AuthenticationFailedException ,
    MessagingException
  {
    // Retrieve session object with the mail.smtp.host property
    Properties sessionProperties = new Properties();
    sessionProperties.put("mail.smtp.host", this.SMTPHost);
    mailSession = Session.getDefaultInstance(sessionProperties, null);

    // Get a message store that implements the protocol specified
    mailStore = mailSession.getStore(this.MSG_STORE_MAIL);

    // Connects to message store host/port with user name and password
    mailStore.connect(MessageStoreHost, MessageStorePort, userName, password);
    System.out.println("Mail Store :"+mailStore);
  }

  public void readMessages()
    throws MessagingException
  {
    createFolder();
    if( openFolder() == 0 ) return; // Return if no messages

    // Retrieve the messages
    Message[]  newMessages = mailFolder.getMessages();
    // Print each message that hasn't been seen
    for (int i = 0; i < newMessages.length; i++)
    {
      if (!newMessages[i].isSet(Flags.Flag.SEEN)){
        printMessage(newMessages[i]);
        // Mark message to delete
        newMessages[i].setFlag(Flags.Flag.DELETED, true);
      }
    }

    // close folder
    this.mailFolder.close(true);
  }

  private void createFolder()
    throws MessagingException
  {
      // Get INBOX folder
      mailFolder = mailStore.getFolder(INBOX);
      if(mailFolder == null){
         throw new NullPointerException("Unable to get Inbox folder");
      }
  }

  private int openFolder()
    throws MessagingException
  {
    // Open folder for read only access
    mailFolder.open(Folder.READ_ONLY);
    // Get number of inbox messages
    int numberOfMessagesInInbox = mailFolder.getMessageCount();
    
    if(numberOfMessagesInInbox == 0){
      System.out.println("No Messages");
      mailFolder.close(true);
    }

    return numberOfMessagesInInbox;
  }

  public void printMessage( Message  message)
    throws MessagingException
  {
    // Get various data elements from message
    if(message != null){
     // Get various data elements from message
     String subject = message.getSubject();
     Date sentDate = message.getSentDate();
     int messageSize    = message.getSize();
     String[] headers = message.getHeader(this.MAIL_HEADER);
     Address[] mailFrom = message.getFrom();
     Address[] allRecipients = message.getAllRecipients();
     InputStreamReader inputStream = null;
     char[] buf = new char[messageSize];
     String contents = "";
     try{
       inputStream =
          new InputStreamReader(message.getInputStream());
       int len = 0;
       do
       {
          len = inputStream.read(buf, 0, messageSize);
          if (len > 0)
             contents += new String(buf);
       }
       while (len != -1);
     }
     catch(IOException ioException){
      System.out.println("Error: " +ioException);
      return;
     }

     // Print out various elements that were read from message
     System.out.println("Message Subject :"+subject);
     System.out.println("Message Date: "+sentDate);
     System.out.println("messageSize :"+messageSize);
     System.out.println("Headers :");
     for(int i = 0; headers != null && (i<headers.length); i++){
      System.out.print(headers[i]);
     }
     System.out.println("From :");
     for(int i = 0; i < mailFrom.length; i++){
      System.out.print("Address :" +mailFrom[i].toString());
     }
     System.out.println();
     System.out.println("Contents :");
     System.out.println(contents);
     System.out.println("\nEnd Of Contents");
   }
  }

/**
  *   send the message to the user.
  */
  public void sendAMessage()
    throws Exception
  {
     Preferences preferences =
         Preferences.userNodeForPackage(MailClient.class);
      // Read from, to, cc, subject, and body elements from preferences
     String from = preferences.get("MESSAGE_FROM","");
     String messageTo = preferences.get("MESSAGE_TO","");
     String messageCCTo = preferences.get("MESSAGE_CC","");
     String subject = preferences.get("MESSAGE_SUBJECT","");
     String body = preferences.get("MESSAGE_BODY","");
     String attachment1 = preferences.get("ATTACH_PART1","");
     String attachment2 = preferences.get("ATTACH_PART2","");
     
     // construct a message using SMTP session
     MimeMessage message = new MimeMessage(mailSession);
     boolean parseStrict = false;
     // set who message is from
     message.setFrom(InternetAddress.parse(from, parseStrict)[0]);
     // set recipients of message
     message.setRecipients(Message.RecipientType.TO,
                 InternetAddress.parse(messageTo, parseStrict));
     if(messageCCTo.trim().length() != 0){
      message.setRecipients(Message.RecipientType.CC,
                 InternetAddress.parse(messageCCTo, parseStrict));
     }
     // set subject
     message.setSubject(subject);
     // set header
     String mailer = "sendMessage";
     message.setHeader("X-Mailer", mailer);
     // set date
     message.setSentDate(new Date());
     // Body Part Section
     MimeBodyPart mimeBodyPart = new MimeBodyPart();
     mimeBodyPart.setText(body);
     // Another Body Part
     MimeBodyPart mimeBodyPartAnother = new MimeBodyPart();
     mimeBodyPartAnother.setText(attachment1);
     // Another Body Part
     MimeBodyPart mimeBodyPartAnother1 = new MimeBodyPart();
     mimeBodyPartAnother1.setText(attachment2);
     MimeMultipart multiPart = new MimeMultipart();
     multiPart.addBodyPart(mimeBodyPart);
     multiPart.addBodyPart(mimeBodyPartAnother);
     multiPart.addBodyPart(mimeBodyPartAnother1);
     
     // set body
     message.setContent(multiPart);
     // send message
     Transport.send(message);
     System.out.println("\nMail was sent successfully.");

  }

  /**
  * Search messages given a from address and subject
  */
  public void  searchFrom(String fromAddress, String subject)
    throws MessagingException
  {
     //
    createFolder();
    if( openFolder() == 0 ) return; // Return if no messages

    // Create a new subject term
    SearchTerm searchTerm = new SubjectTerm(subject);
    // Create a new 'from' term
    FromStringTerm fromStringTerm = new FromStringTerm(fromAddress);
    // Combine from and search term
    searchTerm = new OrTerm(searchTerm, fromStringTerm);
    // Create a new date term (today's)
    ReceivedDateTerm receivedDateTerm =
      new ReceivedDateTerm(ComparisonTerm.EQ, new Date());
    // Combine date term
    searchTerm = new OrTerm(searchTerm,receivedDateTerm);
    // Induce search for messages
    Message[]  newMessages = mailFolder.search(searchTerm);
    if(newMessages == null || newMessages.length == 0){
      System.out.println(" no message matches search Criteria");
    }
    // Print each found message
    for (int i = 0; i < newMessages.length; i++)
    {
      if (!newMessages[i].isSet(Flags.Flag.SEEN)){
        printMessage(newMessages[i]);
      }
    }

    // Close folder and store
    this.mailFolder.close(true);
    this.mailStore.close();
  }



  public static void main(String[] args)
  {
    
    Preferences preferences =
      Preferences.userNodeForPackage(
        ejava.javamail.MailClient.class);

    // Create new MailClient
    MailClient mailClient = new MailClient();
    try{
      // Connect to mail server
      mailClient.connectToMailServer();

      // Read messages from server
      mailClient.readMessages();

      // Send a message to the server with property data
      mailClient.sendAMessage();
        
      String from = preferences.get("MESSAGE_FROM","");
      String subject = preferences.get("MESSAGE_SUBJECT","");
      // Search for E-mail with matching from and subject fields
      mailClient.searchFrom(from,subject);
    }
    catch(MessagingException messagingException){
      System.out.println("Messaging Error :"+messagingException);
      messagingException.printStackTrace();
    }
    catch(Exception exception){
      System.out.println(" Error :" + exception);
      exception.printStackTrace();
    }
  }
}


