package ejava.jaxp;

import org.xml.sax.SAXException;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXParseException;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;

import java.io.*;

/**
  * An example to induce the parsing of an XML document
  */
public class SaxExample
{

  static final String SCHEMA_PROP =
              "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
  static final String SCHEMA_PROP_VALUE =
              "http://www.w3.org/2001/XMLSchema";
  static final String SCHEMA_SOURCE_PROP =
              "http://java.sun.com/xml/jaxp/properties/schemaSource";

  public static void main(String[] argv)
  {
    if(argv.length < 1){
      System.out.print("Usage: Specify XML and optional schema file.");
      System.out.println("java ejava.jaxp.SaxExample <xml> [schema]");
      System.exit(0);
    }
    try {
      String xmlFileURI = "file:" + new File(argv[0]).getAbsolutePath();
      // Get Parser Factory
      SAXParserFactory saxParserFactory = 
          SAXParserFactory.newInstance();
      // set Document Validation true
      saxParserFactory.setValidating(true);
      // set Namespace Awareness true
      saxParserFactory.setNamespaceAware(true);

      // get SAX parser from Factory
      SAXParser saxParser = saxParserFactory.newSAXParser();
          
      // if using a schema for validation, then set schema 
      // property and source
      if(argv.length == 2){
          saxParser.setProperty(SCHEMA_PROP, SCHEMA_PROP_VALUE);
          saxParser.setProperty(SCHEMA_SOURCE_PROP, new File(argv[1]));
      }    
      
      // get an XMLReader from SAX parser
      XMLReader reader = saxParser.getXMLReader();
      // set Content Handler to receive document Handler Events
      reader.setContentHandler(new SampleContentHandler());
      // set DTD Handler to receive DTD events from Parser
      reader.setDTDHandler(new SampleDTDHandler());
      // set EntityResolver to receive EntityResolver Events.
      reader.setEntityResolver(new SampleEntityResolver());
      // set ErrorHandler to receive Errors and warnings.
      reader.setErrorHandler(new SampleErrorHandler());
      // parse document
      reader.parse(xmlFileURI);
    }
    catch (SAXParseException saxParseException) {
      System.out.println (" Parsing error"
             + ", at line " + saxParseException.getLineNumber ()
             + ", in file  " + saxParseException.getSystemId ());
      System.out.println("   " + saxParseException.getMessage ());
      saxParseException.printStackTrace();
    }
    catch (SAXException saxException) {
      saxException.printStackTrace ();
    }
    catch (Throwable throwable) {
      throwable.printStackTrace ();
    }
  }
} 
