package ejava.jdbc.core;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.prefs.Preferences;

public class ResultSetExample extends SimpleStatementExample
{
    private ResultSet resultSet; // ResultSet Object
    
    public ResultSetExample()
    {
        super();
    }
    
    /**
     Provides some key information from ResultSetMetaData
     */
    public static void analyzeMetaData(ResultSet resultSet)
        throws SQLException
    {
        ResultSetMetaData rMetaData = resultSet.getMetaData();
        
        for(int i = 1; i<= rMetaData.getColumnCount(); i++){
            System.out.println("Column Name : "+
                rMetaData.getColumnName(i));
            System.out.println("Column Label :  "+
                rMetaData.getColumnLabel(i));
            System.out.println("Column Display Size : "+
                rMetaData.getColumnDisplaySize(i));
            int type = rMetaData.getColumnType(i);
            System.out.println("Column Java Type :  " +
                getColumnClass(type));
            System.out.println("Column Type Name :  " +
                rMetaData.getColumnTypeName(i));
            System.out.println("Null Value Allowed :  " +
                rMetaData.isNullable(i));
            System.out.println("Is Read Only :  " +
                rMetaData.isReadOnly(i));
        }
    }
    
    /**
     Parses a ResultSet and prints out java.sql.Types type, converted
     Java type, and value.
     */
    public static void parseResultData(ResultSet resultSet)
        throws SQLException
    {
        ResultSetMetaData rMetaData = resultSet.getMetaData();
        int nColumns = rMetaData.getColumnCount();
        
        while(resultSet.next()){
            for(int i = 1; i <= nColumns; i++ ){
                int type =  rMetaData.getColumnType(i);
                switch(type) {
                    case Types.CHAR:
                        System.out.print("(CHAR->String "
                            + resultSet.getString(i) + "):");
                        break;
                    case Types.VARCHAR:
                        System.out.print("(VARCHAR->String "
                            + resultSet.getString(i) + "):");
                        break;
                    case Types.LONGVARCHAR:
                        System.out.print("(LONGVARCHAR->String "
                            + resultSet.getString(i) + "):");
                        break;
                    case Types.BIT:
                        System.out.print("(BIT->byte "
                            + resultSet.getByte(i)+ "):");
                        break;
                    case Types.TINYINT:
                        System.out.print("(TINYINT->int "
                            + resultSet.getInt(i)+ "):");
                        break;
                    case Types.SMALLINT:
                        System.out.print("(SMALLINT->int "
                            + resultSet.getInt(i)+ "):");
                        break;
                    case Types.INTEGER:
                        System.out.print("(INTEGER->int "
                            + resultSet.getInt(i)+ "):");
                        break;
                    case Types.BIGINT:
                        System.out.print("(BIGINT->long "
                            + resultSet.getLong(i)+ "):");
                        break;
                    case Types.FLOAT:
                        System.out.print("(FLOAT->float "
                            + resultSet.getFloat(i)+ "):");
                        break;
                    case Types.DOUBLE:
                        System.out.print("(DOUBLE->double "
                            + resultSet.getDouble(i)+ "):");
                        break;
                    case Types.DATE:
                        System.out.print("(DATE->Date "
                            + resultSet.getDate(i)+ "):");
                        break;
                    case Types.TIME:
                        System.out.print("(TIME->Time "
                            + resultSet.getTime(i)+ "):");
                        break;
                    case Types.TIMESTAMP:
                        System.out.print("(TIMESTAP->Timestamp "
                            + resultSet.getTimestamp(i)+ "):");
                        break;
                    default:
                        System.out.print("(<other type>->Object "
                            + resultSet.getObject(i)+ "):");
                        break;
                }
            }
            System.out.println();
        }
    }
    
    /**
     Returns the column Class given a java.sql.Types value.
     */
    public static Class getColumnClass(int type)
    {
        switch(type) {
            case Types.CHAR:
            case Types.VARCHAR:
            case Types.LONGVARCHAR:
                return String.class;
                
            case Types.BIT:
                return Boolean.class;
                
            case Types.TINYINT:
            case Types.SMALLINT:
            case Types.INTEGER:
                return Integer.class;
                
            case Types.BIGINT:
                return Long.class;
                
            case Types.FLOAT:
            case Types.DOUBLE:
                return Double.class;
                
            case Types.DATE:
                return java.sql.Date.class;
                
            default:
                return Object.class;
        }
    }
    
    /**
     Prints ResultSet to the standard ouput stream.
     */
    public void printResultSet(ResultSet resultSet)
        throws SQLException
    {
        ResultSetMetaData resultSetMetaData = resultSet.getMetaData();
        
        int nColumns =  resultSetMetaData.getColumnCount();
        
        for(int col= 1; col <= nColumns; col++){
            System.out.print(resultSetMetaData.getColumnName(col)
                + "  :   ");
        }
        
        System.out.println();
        while(resultSet.next()){
            for(int col= 1; col <= nColumns; col++){
                System.out.print(resultSet.getObject(col)  +  " : ");
            }
            System.out.println();
        }
    }
    
    public static void main(String[] args)
    {
        Preferences preferences =
            Preferences.userNodeForPackage(
            ejava.jdbc.core.ConnectToDatabase.class);
        
        ResultSetExample resultSetExample = new ResultSetExample();
        try{
            
            Connection connection = resultSetExample.getConnection();
            // Get SQL from preferences, execute, get and view results...
            
            String sqlString 
                = preferences.get("SQL_QUERY_STATEMENT",null);
            ResultSet resultSet 
                = resultSetExample.executeQuery(sqlString);
            // wait for user input.
            ResultSetExample.analyzeMetaData(resultSet);
            System.out.println("Enter when Ready to see the ResultSet "
                +" Results and each field type for statement :"
                +sqlString);
            // wait for user input.
            System.in.read();
            ResultSetExample.parseResultData(resultSet);
            resultSetExample.closeStatement();
        }
        catch(ClassNotFoundException classNotFoundException){
            String driverClass = preferences.get("DRIVER_CLASS",null);
            System.out.println( "Driver Class :" +driverClass +
                " Exception :"+classNotFoundException);
            System.exit(0);
        }
        catch(SQLException sqlExcetpion){
            String dbURL = preferences.get("DATABASE_URL",null);
            System.out.println( "SQLException for db :" +dbURL+
                " Exception :"+sqlExcetpion);
            System.exit(0);
        }
        catch(Exception exception){
            System.out.println( "General exception :"+exception);
            exception.printStackTrace();
            System.exit(0);
        }
        
        System.exit(0);
    }
}
