package ejava.jdbc.core;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Connection;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.prefs.Preferences;

/**
 This exmple gives an overview of the PreparedStatement object and how to
 use frequently used methods.
 */

public class SimplePreparedStatementExample extends ConnectToDatabase
{
	private PreparedStatement statement; // PreparedStatement object
	
	public SimplePreparedStatementExample()
	{
		super();
	}
	
	/**
	 If not connected to database...connect to database,
	 create a statement,	 and return the PreparedStatement.
	 */
	protected void createStatement(String sqlString)
		throws SQLException, Exception
	{
		Connection connection = super.getConnection();
		statement = connection.prepareStatement(sqlString);
	}
	
	/**
	 This method executes a statement and returns true or false. Any type
	 of PreparedStatement may be executed by this command.
	 */
	public boolean execute(String sqlString, Vector values)
		throws SQLException, Exception
	{
		
		createStatement(sqlString);
		// Set the values for the ? marks in the SQL Statement.
		for(int i = 1; i <= values.size(); i++){
			statement.setObject(i,values.elementAt(i-1));
		}
		
		boolean returnValue = statement.execute();
		
		return returnValue;
	}
	
	/**
	 This method executes the statement and returns a ResultSet from
	 execution of the query. Only a SELECT statement is applicable here.
	 */
	public ResultSet executeQuery(String sqlString, Vector values)
		throws SQLException, Exception
	{
		createStatement(sqlString);
		// Set the values for the ? marks in the SQL Statement.
		for(int i = 1; i <= values.size(); i++){
			statement.setObject(i,values.elementAt(i-1));
		}
		
		ResultSet rs = statement.executeQuery();
		return rs;
	}
	
	/**
	 This method executes update statements and returns a count of
	 the updates. Only a INSERTS, DELETES, and UPDATES are applicable
	 here.In addition, SQL statements that return nothing such as SQL DDL
	 statements	 can be executed.
	 */
	public int executeUpdate(String sqlString, Vector values)
		throws SQLException, Exception
	{
		
		createStatement(sqlString);
		
		// Set the values for the ? marks in the SQL Statement.
		for(int i = 1; i <= values.size(); i++){
			statement.setObject(i,values.elementAt(i-1));
		}
		
		int returnValue = statement.executeUpdate();
		
		return returnValue;
	}
	
	/**
	 This method closes the Statement.
	 */
	public void closeStatement()
		throws SQLException
	{
		if(statement != null){
			statement.close();
			statement = null;
		}
	}
	
	/**
	 Convenience method for parsing a String into a Vector.
	 */
	public static Vector parseStringToVector(String s)
	{
		Vector v = new Vector();
		StringTokenizer st = new StringTokenizer(s,",");
		int i = 0;
		while(st.hasMoreTokens()){
			String nthToken = (String)st.nextElement();
			v.addElement(nthToken);
		}
		
		return v;
	}
	
	public static void main(String[] args)
	{
		Preferences preferences =
			Preferences.userNodeForPackage(
			ejava.jdbc.core.ConnectToDatabase.class);
		
		SimplePreparedStatementExample simplePreparedStatementExample
			= new SimplePreparedStatementExample();
		try{
			// Establish properties from preferences and get connection
			Connection connection =
				simplePreparedStatementExample.getConnection();
				
			// Get SQL statements from preferences  and execute them...
			
			// Get statement info
			String sqlString  =
				preferences.get("PREPARED_SQL_QUERY_STATEMENT",null);
			String parameters =
				preferences.get("PREPARED_SQL_QUERY_STATEMENT_VALUES",null);
			System.out.println("Executing statement :"+sqlString);
			Vector values =
				simplePreparedStatementExample.
					parseStringToVector(parameters);

			// Issue an execute() command
			boolean executionStatus =
				simplePreparedStatementExample.execute(sqlString,values);
			if(executionStatus == false){
				System.out.println("Failed to Execute The Statement :" +
					sqlString);
			}
			
			// Set SQL insert string and values
			String insertString =
				preferences.get("PREPARED_SQL_INSERT_STATEMENT",null);
			System.out.println("Executing statement :"+insertString);
			String insertValuesString =
				preferences.get("PREPARED_SQL_INSERT_STATEMENT_VALUES",null);
			values = simplePreparedStatementExample.
				parseStringToVector(insertValuesString);
			// Issue an executeUpdate command
			int nResults = simplePreparedStatementExample.
				executeUpdate(insertString, values);
			
			// Set SQL update string and values
			String updateString =
				preferences.get("PREPARED_SQL_UPDATE_STATEMENT",null);
			System.out.println("Executing statement :"+updateString);
			String updateValuesString =
				preferences.get("PREPARED_SQL_UPDATE_STATEMENT_VALUES",null);
			values =  simplePreparedStatementExample.
				parseStringToVector(updateValuesString);
			// Issue an executeUpdate command
			nResults =  simplePreparedStatementExample.
				executeUpdate(updateString, values);
			
			// Close the statement
			simplePreparedStatementExample.closeStatement();
		}
		catch(ClassNotFoundException classNotFoundException){
			String driverClass = preferences.get("DRIVER_CLASS",null);
			System.out.println( "Driver Class :" +driverClass +
				" Exception :"+classNotFoundException);
			System.exit(0);
		}
		catch(SQLException sqlExcetpion){
			String dbURL = preferences.get("DATABASE_URL",null);
			System.out.println( "SQLException with db :" +dbURL+
				" Exception :"+sqlExcetpion);
			System.exit(0);
		}
		catch(Exception exception){
			System.out.println( "General Exception :"+exception);
			System.exit(0);
		}
		
		try{
			String sqlString  =
				preferences.get("PREPARED_SQL_QUERY_STATEMENT",null);
			String parameters =
				preferences.get("PREPARED_SQL_QUERY_STATEMENT_VALUES",null);
			System.out.println("Executing statement :"+sqlString);
			Vector values =
				simplePreparedStatementExample.
					parseStringToVector(parameters);
		
			ResultSet rs = simplePreparedStatementExample.
				executeQuery(sqlString,values);
			
			// Close the statement
			simplePreparedStatementExample.closeStatement();
		}
		catch(SQLException sqlExcetpion){
			String dbURL = preferences.get("DATABASE_URL",null);
			System.out.println( "SQLException with db :" +dbURL+
				" Exception :"+sqlExcetpion);
			System.exit(0);
		}
		catch(Exception exception){
			System.out.println( "General Exception :"+exception);
			System.exit(0);
		}
		
		System.exit(0);
	}
}
