package ejava.jms;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.util.Hashtable;
import java.util.Properties;
import java.util.prefs.Preferences;
/**
  * This class is is a helper class to be used by the QueueSupplier
  * and TopicSupplier
  */
public class OrderManager
{
/**
  * This method reads inventory information of shirts for each
  * supplier, if the shirts from a supplier is lower than
  * the the max limit to order, it creates an OrderRequest Object
  * to that supplier and returns.
  */
  public static Hashtable getNewOrders()
    throws SQLException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
   // Connect to database
    String dbDriverClassName = preferences.get("DRIVER_CLASS", null);
    String dbURL = preferences.get("DATABASE_URL", null);
    String userName = preferences.get("USER_NAME", null);
    String password = preferences.get("PASSWORD", null);

    Connection connection = null;

   
    try {
      Class.forName(dbDriverClassName);
      if (userName == null || password == null || userName.length() == 0
              || password.length() == 0) {
        connection = DriverManager.getConnection(dbURL);
      } else {
        connection = DriverManager.getConnection(dbURL, userName,
                                                 password);
        connection.setAutoCommit(false);
      }
    }
    catch (ClassNotFoundException classNotFoundException) {
      System.out.println("Driver Class Not found " +
      classNotFoundException);
      return null;
    }
    catch (SQLException sqlException) {
      System.out.println("SQL Exception :" + sqlException);
      return null;
    }
   // Get distinct suppliers
   String suppliersSQLString =
    preferences.get("DISTINCT_SUPPLIERS",null);
  
     Statement statement = connection.createStatement();
     String availableShirtsSQLString =
        preferences.get("AVAILABLE_SHIRTS",null);
     PreparedStatement preparedStatement
        = connection.prepareStatement(availableShirtsSQLString);
     String newInventoryRequestSQLString =
        preferences.get("NEW_INVENTORY_REQUEST",null);
     PreparedStatement preparedStatementForNewInvotoryRequest
        = connection.prepareStatement(newInventoryRequestSQLString);
     int maxLimitToOrder = preferences.getInt("MAX_LIMIT_TO_ORDER",-1);

     int nextOrderQuentity = preferences.getInt("ORDER_QUANTITY",-1);

     // get the distinct suppliers
     ResultSet resultSet = statement.executeQuery(suppliersSQLString);
     Hashtable returnRequests = new Hashtable();

     while(resultSet.next()){
      int supplierID = resultSet.getInt(1);
      //for each supplier get the available shirts
      preparedStatement.setInt(1,supplierID);
      ResultSet rs = preparedStatement.executeQuery();
      int availableShirts = -1;
      while(rs.next()){
        availableShirts = rs.getInt(1);
      }
      rs.close();

      // if the available shirts are less than the maxLimitToOrder,
      // then we need to order those shirts for the new customer
      if(availableShirts < maxLimitToOrder){
        preparedStatementForNewInvotoryRequest.setInt(1,supplierID);
        rs = preparedStatementForNewInvotoryRequest.executeQuery();
        while(rs.next()){
          OrderRequest orderRequest = new OrderRequest();
          orderRequest.color = rs.getString("COLOR");
          orderRequest.size = rs.getString("SHIRT_SIZE");
          orderRequest.designBack =  rs.getString("DESIGN_BACK");
          orderRequest.designFront = rs.getString("DESIGN_FRONT");
          orderRequest.howMany = nextOrderQuentity;
          String queueName = rs.getString("NAME");
          returnRequests.put(queueName,orderRequest);
        }
        rs.close();
      }
     }
     preparedStatement.close();
     preparedStatementForNewInvotoryRequest.close();
     statement.close();
     connection.close();

     return returnRequests;
  }
}
