package  ejava.jms;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.jms.JMSException;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.QueueConnectionFactory;
import javax.jms.Queue;
import javax.jms.TextMessage;
import javax.jms.BytesMessage;
import javax.jms.ObjectMessage;
import javax.jms.StreamMessage;
import javax.naming.NamingException;
import java.util.Properties;
import java.util.Hashtable;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.jms.Session;
import javax.jms.DeliveryMode;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Date;
import java.util.prefs.Preferences;

/**
  * This example shows how to use JMS,
  * in queue mode and receive a message from the queue.
  */
public class QueueConsumer   implements MessageListener
{
  private QueueConnectionFactory queueConnectionFactory;
  private QueueConnection queueConnection;
  private QueueSession queueSession;
  private QueueReceiver queueReceiver;
  private Queue queue;
  public  boolean quitFromReceiving = false;

/**
  * Constructor, creates all the Objects to receive messages
  */
  public QueueConsumer( Context context)
    throws NamingException, JMSException
  {
    
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
    String jmsFactoryName =
         preferences.get("JMS_FACTORY_FOR_QUEUE",null);
    // Create Queue Connection Factory
    queueConnectionFactory = (QueueConnectionFactory)
      context.lookup(jmsFactoryName);
    // Create Queue Connection to The Factory
    queueConnection = queueConnectionFactory.createQueueConnection();
    // Create Session to the Connection
    queueSession = queueConnection.createQueueSession(false,
      Session.AUTO_ACKNOWLEDGE);
    String queueName = preferences.get("QUEUE_NAME",null);
    try {
      queue = (Queue) context.lookup(queueName);
    } catch (NamingException namingException) {
      // if not created, create new queue
      queue = queueSession.createQueue(queueName);
      // bind the queue with a name
      context.bind(queueName, queue);
    }
    // Create Receiver
    queueReceiver = queueSession.createReceiver(queue);
    // Register QueueConsumer as Message Listener
    queueReceiver.setMessageListener(this);
    // Start Receiving Message
    queueConnection.start();
  }

/**
  * implements onMessage, onMessage will be invoked by the Queue
  * whenever it need to send a message to the receiver.
  */
  public void onMessage(Message message)
  {
    try {
      // print information about the message
      System.out.println(message.getJMSMessageID());
      System.out.println("MessageID :" + message.getJMSMessageID() +
                           " for " + message.getJMSDestination());
      System.out.print("Message Expires        ");
      if (message.getJMSExpiration() > 0) {
          System.out.println( new Date( message.getJMSExpiration()));
      }
      else{
          System.out.println("never  Expires ");
      }
      System.out.println("Priority :" + message.getJMSPriority());
      System.out.println("Mode   : " + (
        message.getJMSDeliveryMode() == DeliveryMode.PERSISTENT ?
        "PERSISTENT" : "NON_PERSISTENT"));
      System.out.println("Reply to :      " + message.getJMSReplyTo());
      System.out.println("Message type :  " + message.getJMSType());
      if (message instanceof TextMessage) {

        String receivedMessage = ((TextMessage)message).getText();
        System.out.println("Received Message :" + receivedMessage);

      } else if(message instanceof ObjectMessage){

        String receivedMessage = message.toString();
        System.out.println("Received Message :" + receivedMessage);

      }
      
      quitFromReceiving = true;
    }
    catch (JMSException jmsException) {
      jmsException.printStackTrace();
    }
  }
/**
  * Close JMS Reletated Object.
  */
  public void close()
       throws JMSException
  {
    this.queueReceiver.close();
    this.queueSession.close();
    this.queueConnection.close();
  }
/**
  * returns the initial Context to the Queue Service
  */
  private static InitialContext getInitialContext()
       throws NamingException
  {
    System.out.println(" Getting Initial Context :");
   // Connect to naming Service
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
    Hashtable env = new Hashtable();
    String jndiFactory = preferences.get("JNDI_FACTORY", null);
    String providerURL  = preferences.get("JNDI_PROVIDER_URL", null);
    env.put(Context.INITIAL_CONTEXT_FACTORY, jndiFactory);
    env.put(Context.PROVIDER_URL, providerURL);
    return new InitialContext(env);
  }

   public static void main(String[] args)
       throws Exception
  {
    try{
      // Create initial context and queue consumer
      InitialContext context = getInitialContext();
      QueueConsumer queueConsumer = new QueueConsumer(context);
      System.out.println(" Queue Client is ready to receive Message");
      // Wait for messages to receive, until user quits from  program.
      synchronized(queueConsumer) {
        while (! queueConsumer.quitFromReceiving) {
          try {
            queueConsumer.wait();
          }
          catch (InterruptedException interruptedException){
            System.out.println(" Error :"+ interruptedException);
            interruptedException.printStackTrace();
          }
        }
      }
      queueConsumer.close();
      context.close();
      System.exit(0);
    }
    catch(NamingException namingException){
    
      System.out.println("Naming Exception :"+namingException);
      namingException.printStackTrace();
    }
  }


}




