package  ejava.jms;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.QueueSender;
import javax.jms.Queue;
import java.util.Properties;
import java.util.Hashtable;
import javax.jms.JMSException;
import javax.naming.NamingException;
import javax.jms.Message;
import javax.jms.TextMessage;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import java.io.Serializable;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.prefs.Preferences;
/**
  * This class implements the QueueSupplier to notify
  * the TSHIRTS Suppliers of new Orders.
  */
public class QueueSupplier
{
  private QueueConnectionFactory queueConnectionFactory;
  private QueueConnection queueConnection;
  private QueueSession queueSession;
  private QueueSender queueSender;
  private Queue queue;
  private Properties properties;
  public  boolean quitFromReceiving = false;
  
/**
  * Constructor , it initializes  all the  Objects to send
  * messages to Queue.
  */
  public QueueSupplier( Context context,
    String queueName)
    throws NamingException, JMSException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
    String jmsFactoryName =
         preferences.get("JMS_FACTORY_FOR_QUEUE",null);
    // Create Queue Connection Factory
    queueConnectionFactory = (QueueConnectionFactory)
      context.lookup(jmsFactoryName);
    // Create Queue Connection to The Factory
    queueConnection = queueConnectionFactory.createQueueConnection();
    // Create Session to the Connection
    queueSession = queueConnection.createQueueSession(false,
      Session.AUTO_ACKNOWLEDGE);
    try {
      // if queue already created
      queue = (Queue) context.lookup(queueName);
    } catch (NamingException namingException) {
      // if needed to create new Queue
      queue = queueSession.createQueue(queueName);
      // bind the queue with a name
      context.bind(queueName, queue);
    }
    // Create Receiver
    queueSender = queueSession.createSender(queue);
  }

/**
  * Send a message to a  Queue.
  */
  public void send(Object newMessage)
       throws JMSException
  {
    System.out.println(" Sending :"+newMessage);
    if(newMessage instanceof String){
      TextMessage sendingMessage = queueSession.createTextMessage();
      queueConnection.start();
      sendingMessage.setText((String)newMessage);
      queueSender.send(sendingMessage);
    }else {
      ObjectMessage sendingMessage = queueSession.createObjectMessage();
      queueConnection.start();
      sendingMessage.setObject((Serializable)newMessage);
      queueSender.send(sendingMessage);
    }
  }
/**
  * Close JMS Reletated Objects.
  */
  public void close()
       throws JMSException
  {
    queueSender.close();
    queueSession.close();
    queueConnection.close();
  }


/**
  * Create a new Queue and send the Order to that queue
  */

  private static void createAQueueAndSendOrder(
    String queueName, OrderRequest orderRequest)
  {
    try{
      InitialContext context = getInitialContext();
      QueueSupplier queueSupplier =
        new QueueSupplier(context, queueName);
      queueSupplier.send(orderRequest);
      queueSupplier.close();
      context.close();
    }
    catch(JMSException jmsException){
      System.out.println("Error :"+jmsException);
      jmsException.printStackTrace();
    }
    catch(NamingException namingException){
      System.out.println("Error :"+namingException);
      namingException.printStackTrace();
    }
  }
/**
  * returns the initial Context to the Queue Service
  */
  private static InitialContext getInitialContext()
       throws NamingException
  {
    System.out.println(" Getting Initial Context :");
   // Connect to naming Service
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
    Hashtable env = new Hashtable();
    String jndiFactory = preferences.get("JNDI_FACTORY", null);
    String providerURL  = preferences.get("JNDI_PROVIDER_URL", null);
    env.put(Context.INITIAL_CONTEXT_FACTORY, jndiFactory);
    env.put(Context.PROVIDER_URL, providerURL);
    return new InitialContext(env);
  }

  public static void main(String[] args)
  {
    try{
      // Get new orders from database using order manager class
      Hashtable newOrders = OrderManager.getNewOrders();
      Enumeration keys = newOrders.keys();
      // For each order, get order request and send to consumer
      while(keys.hasMoreElements()){
        String queueName = (String)keys.nextElement();
        OrderRequest orderRequest =
          (OrderRequest)newOrders.get(queueName);
        createAQueueAndSendOrder(queueName,orderRequest);
      }

    }
    catch(SQLException sqlException){
      System.out.println("Error :"+sqlException);
      sqlException.printStackTrace();
    }

  }
}










