package  ejava.jms;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import java.util.Properties;
import java.util.Hashtable;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.jms.TopicPublisher;
import javax.jms.Topic;
import javax.jms.TextMessage;
import javax.jms.ObjectMessage;
import javax.jms.JMSException;
import javax.jms.Session;
import java.io.Serializable;
import javax.jms.MessageListener;
import javax.jms.Message;
import javax.jms.StreamMessage;
import javax.jms.BytesMessage;
import java.io.IOException;
import java.io.FileInputStream;
import javax.jms.DeliveryMode;
import java.util.Date;
import java.util.prefs.Preferences;

/**
  * This example shows how to use JMS,
  * in Topic mode and  receive messages from the Topic.
  */
public class TopicReplyConsumer  implements MessageListener
{
  private TopicConnectionFactory topicConnectionFactory;
  private TopicConnection topicConnection;
  private TopicSession topicSession;
  private TopicSubscriber topicSubscriber;
  private Topic topic;
  private Properties properties;
  public  boolean quitFromReceiving = false;

/**
  * Constructor , it creates all the needed Objects to make receive
  * messages.
  */
  public TopicReplyConsumer( Context context)
    throws NamingException, JMSException
  {
     Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
    String jmsFactoryName = preferences.get("JMS_FACTORY_FOR_TOPIC",null);
    // Create Queue Connection Factory
    topicConnectionFactory = (TopicConnectionFactory)
      context.lookup(jmsFactoryName);
    // Create Queue Connection to The Factory
    topicConnection = topicConnectionFactory.createTopicConnection();
    // Create Session to the Connection
    topicSession = topicConnection.createTopicSession(false,
      Session.AUTO_ACKNOWLEDGE);
    String topicName = preferences.get("TOPIC_NAME",null);
    try {
      topic = (Topic) context.lookup(topicName);
    } catch (NamingException namingException) {
      // create topic, if not created
      topic = topicSession.createTopic(topicName);
      // bind the topic with a name
      context.bind(topicName, topic);
    }
    // Create Receiver
    topicSubscriber = topicSession.createSubscriber(topic);
    // Register TopicClient as Message Listener
    topicSubscriber.setMessageListener(this);
    // Start Receiving Message
    topicConnection.start();
    // start receiving
  }

  /**
  * implements onMessage , the onMessage will be invoked by the Queue
  * whenever it need to send the message to the receiver.
  */
  public void onMessage(Message message)
  {
    try {
      if(message instanceof ObjectMessage){

        String receivedMessage = message.toString();
        System.out.println("Received Message :" + receivedMessage);
        ObjectMessage objectMessage = (ObjectMessage)message;
        OrderRequest orderRequest =
            (OrderRequest)objectMessage.getObject();
        if(orderRequest.howMany >5){
          System.out.println("Sending reply ");
          sendReply(message);
        }
      }
      quitFromReceiving = true;
      //notifyAll(); // notify to main Thread to leave from the program
    } catch (JMSException jmsException) {
      jmsException.printStackTrace();
    }
  }

  private void  sendReply(Message message)
    throws JMSException
  {
    System.out.println("Received Request:");
    if(message.getJMSReplyTo() == null){
      System.out.println("Nothing to reply......");
      return;
    }
    Topic replyTopic = (Topic)message.getJMSReplyTo();
    System.out.println("Replied on :"+replyTopic);
    TopicPublisher  topicPublisher =
        topicSession.createPublisher(replyTopic);
    TextMessage textMessage =
        topicSession.createTextMessage(
         " We can deliver these shirts in 3 days"+
         " at your eastern location");
    textMessage.setStringProperty("Reagion","Eastern");
    textMessage.setIntProperty("DeliveryDays",3);
    topicPublisher.publish(textMessage);
    textMessage =
        topicSession.createTextMessage(
         " We can deliver these shirts in 3 days"+
         " at your western location");
    textMessage.setStringProperty("Reagion","Western");
    textMessage.setIntProperty("DeliveryDays",3);
  }
/**
  * Close JMS Reletated Object.
  */
  public void close()
       throws JMSException
  {
    topicSubscriber.close();
    this.topicSession.close();
    this.topicConnection.close();
  }
/**
  * it returns the initial Context to the Queue Service
  */
  private static InitialContext getInitialContext()
       throws NamingException
  {
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
     Hashtable env = new Hashtable();
    String jndiFactory = preferences.get("JNDI_FACTORY", null);
    String providerURL  = preferences.get("JNDI_PROVIDER_URL", null);
    env.put(Context.INITIAL_CONTEXT_FACTORY, jndiFactory);
    env.put(Context.PROVIDER_URL, providerURL);
    return new InitialContext(env);
  }

   public static void main(String[] args)
       throws Exception
  {
    try{
      // Create context and topic consumer
      InitialContext context = getInitialContext();
      TopicReplyConsumer topicConsumer = new TopicReplyConsumer(context);
      System.out.println(" Topic Client is ready to receive Message");
      // Wait for messages to be received, until user quits from  program.
      synchronized(topicConsumer) {
        while (! topicConsumer.quitFromReceiving) {
          try {
            topicConsumer.wait();
          }
          catch (InterruptedException interruptedException){

            System.out.println(" Error :"+ interruptedException);
            interruptedException.printStackTrace();
          }
        }
      }
      //System.in.read();
      topicConsumer.close();
      context.close();
      System.exit(0);
    }
    catch(NamingException namingException){

      System.out.println("Naming Exception :"+namingException);
      namingException.printStackTrace();
    }
  }



}






