package ejava.jms;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.Topic;
import javax.jms.TextMessage;
import javax.jms.ObjectMessage;
import javax.jms.Message;
import javax.jms.JMSException;
import javax.jms.Session;
import java.sql.SQLException;
import java.io.Serializable;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.prefs.Preferences;
import java.util.Properties;
import java.util.Hashtable;

/**
  * This class gets orders from the database and
  * submits new orders on a given topic to the consumer.
  */
public class TopicRequestSupplier
{
  private TopicConnectionFactory topicConnectionFactory;
  private TopicConnection topicConnection;
  private TopicSession topicSession;
  private TopicPublisher topicPublisher;
  private Topic topic;
  private Properties properties;


  public TopicRequestSupplier(Context context)
    throws NamingException, JMSException
  {
     Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);

    String jmsFactoryName =
        preferences.get("JMS_FACTORY_FOR_TOPIC",null);
    // get TopicFactory
    topicConnectionFactory = (TopicConnectionFactory)
      context.lookup(jmsFactoryName);
    
    // Get TopicConnection
    topicConnection = topicConnectionFactory.createTopicConnection();
    // Get Session for the Connection
    topicSession =
        topicConnection.createTopicSession(false,
          Session.AUTO_ACKNOWLEDGE);
    String topicName = preferences.get("TOPIC_NAME",null);
    try {
      // determine if topic by this name already exists
      topic = (Topic) context.lookup(topicName);
    } catch (NamingException namingException) {
      // if not there, create new topic
      topic = topicSession.createTopic(topicName);
      // bind the topic with a name
      context.bind(topicName, topic);
    }
    topicPublisher = topicSession.createPublisher(topic);

  }
/**
  * Send a message to a  Topic.
  */
  public void send(Object newMessage)
       throws JMSException
  {
    System.out.println(" Sending :"+newMessage);
    if(newMessage instanceof String){
      TextMessage sendingMessage = topicSession.createTextMessage();
      topicConnection.start();  // start Sending
      sendingMessage.setText((String)newMessage); // set the Text message
      System.out.println("Creating Temp Topic1:");
      Topic tempTopic = topicSession.createTemporaryTopic();
      System.out.println("Temp Topic:1"+tempTopic);
      sendingMessage.setJMSReplyTo(tempTopic);
      topicPublisher.publish(sendingMessage); // send Message
    }else {
      ObjectMessage sendingMessage = topicSession.createObjectMessage();
      topicConnection.start();   // start Sending
      sendingMessage.setObject((OrderRequest)newMessage); // set Object Message
      System.out.println("Creating Temp Topic:");
      Topic tempTopic = topicSession.createTemporaryTopic();
      System.out.println("Temp Topic:"+tempTopic);
      sendingMessage.setJMSReplyTo(tempTopic);
      topicPublisher.publish(sendingMessage); // send message
      readReply(tempTopic);
    }
  }

  public void readReply(Topic tempTopic)
    throws JMSException
  {
      String messageSelect = "DeliveryDays < 5 AND Region like 'Eastern' ";
      TopicSubscriber topicSubscriber =
         topicSession.createSubscriber(tempTopic,messageSelect,false);
      Message replyMessage = topicSubscriber.receive();
      System.out.println("Received Reply message is :"+replyMessage);
  }

/**
  * Close JMS Reletated Objects.
  */
  public void close()
       throws JMSException
  {
    topicPublisher.close();
    topicSession.close();
    topicConnection.close();
  }

/**
  * it returns the initial Context to the Topic Service
  */
  private static InitialContext getInitialContext()
       throws NamingException
  {
    System.out.println(" Getting Initial Context :");
   // Connect to naming Service
    Preferences preferences =
      Preferences.userNodeForPackage(OrderManager.class);
    Hashtable env = new Hashtable();
    String jndiFactory = preferences.get("JNDI_FACTORY", null);
    String providerURL  = preferences.get("JNDI_PROVIDER_URL", null);
    env.put(Context.INITIAL_CONTEXT_FACTORY, jndiFactory);
    env.put(Context.PROVIDER_URL, providerURL);
    return new InitialContext(env);
  }

  /**
  * Create a new Topic Publisher and publish the Order to that Topic
  */
  public static void createATopicAndPublishOrder(OrderRequest orderRequest)
  {
    try{
      InitialContext context = getInitialContext();
      TopicRequestSupplier topicSupplier =
        new TopicRequestSupplier(context);
      topicSupplier.send(orderRequest);
      topicSupplier.close();
      context.close();
    }
    catch(JMSException jmsException){
      System.out.println("Error :"+jmsException);
      jmsException.printStackTrace();
    }
    catch(NamingException namingException){
      System.out.println("Error :"+namingException);
      namingException.printStackTrace();
    }
  }


  public static void main(String[] args)
       throws Exception
  {
    try{
      // Get The Orders if any
      Hashtable newOrders = OrderManager.getNewOrders();
      Enumeration keys = newOrders.keys();
      // For each order, create order request and publish it
      while(keys.hasMoreElements()){
        String  key = (String)keys.nextElement();
        OrderRequest orderRequest =
          (OrderRequest)newOrders.get(key);
        createATopicAndPublishOrder(orderRequest);
      }

    }
    catch(SQLException sqlException){
      System.out.println("Error :"+sqlException);
      sqlException.printStackTrace();
    }

  }
}










