package ejava.jndi.directory;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.Attribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.ModificationItem;
import java.util.Properties;
import java.io.IOException;
import java.io.FileInputStream;
import javax.naming.NamingEnumeration;
import javax.naming.directory.SearchResult;
import java.util.Enumeration;
import javax.naming.directory.SearchControls;

/**
  * To demonstrate the NDS Directory service major functionality
  */
public class SampleSearchExampleNDS
{
  DirContext context = null;
  Properties properties = new Properties();
  String entryDN;

  public SampleSearchExampleNDS(Properties newProperties)
  {
    super();
    properties = newProperties;
    entryDN = (String)properties.get("NDS_ENTRY_DN");
  }

  /**
   * Constructor
   * Naming Client which is initialized with a jndi.properties file.
   * Uncomment appropriate lines for each service provider in
   * jndi.properties.
   *
   * @param propertiesFileName
   *
   * @see
   */
  public SampleSearchExampleNDS(String propertiesFileName)
    throws IOException
  {
    // Open a file input stream for the associated properties
    // fileName.
    FileInputStream propertiesFileStream =
      new FileInputStream(propertiesFileName);
    // Now load the file input stream into the JNDI environment
    // properties.
    // This of course assumes that the file input stream is in a valid
    // Properties format.
    properties.load(propertiesFileStream);
    entryDN = (String)properties.get("NDS_ENTRY_DN");
  }

  /**
   * Method
   *
   *
   * @return
   *
   * @throws NamingException
   *
   * @see
   */
  public DirContext getInitialContext()
    throws NamingException
  {
    // Create initial context if it is not already created
    if (context == null) {
      System.out.println("Properties :" + properties);
        // Read in a set of default paramters.
      context = new InitialDirContext(properties);
      System.out.println("Context is :" + context);
    }
    return context;
  }

  public void printSearchResults(Attributes attrs)
    throws NamingException
  {

      if(attrs == null) {
        System.out.println(" No Attributes :");
      }
      else {
        for(NamingEnumeration ne = attrs.getAll();
          ne.hasMoreElements(); ){

          Attribute attribute = (Attribute)ne.next();
          String id = attribute.getID();
          for (Enumeration vals = attribute.getAll();
            vals.hasMoreElements();){
            System.out.println(id + ": " + vals.nextElement());
          }
        }
      }
  }

  public void printSearchResults(NamingEnumeration enum)
    throws NamingException
  {
    while (enum != null && enum.hasMore()) {
      SearchResult sr = (SearchResult)enum.next();
      System.out.println(" Result Name :" + sr.getName() );
      Attributes attrs = sr.getAttributes();
      if(attrs == null) {
        System.out.println(" No Attributes :");
      }
      else {
        for(NamingEnumeration ne = attrs.getAll();
          ne.hasMoreElements(); ){

          Attribute attribute = (Attribute)ne.next();
          String id = attribute.getID();
          for (Enumeration vals = attribute.getAll();
            vals.hasMoreElements();){
            System.out.println(id + ": " + vals.nextElement());
          }
        }
      }System.out.println();
    }
  }

  public void searchUsingFilterUsingSearch(String filter)
  {
    try{
      DirContext context = getInitialContext();
      SearchControls searchControls = new SearchControls();
      String[] resultAttributes = {"GivenName", "Surname", "PhoneNumber"};
      searchControls.setReturningAttributes(resultAttributes);
      // set the number of reusts
      searchControls.setCountLimit(2);

      // Search for objects with those matching attributes
      NamingEnumeration searchResults
        = context.search(entryDN, filter,
          searchControls);
      printSearchResults(searchResults);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }




  public void addAttributes()
  {
    try{
      DirContext context = getInitialContext();
      ModificationItem modificationItems[] = new ModificationItem[2];
      modificationItems[0] = new ModificationItem(
        DirContext.ADD_ATTRIBUTE,
        new BasicAttribute("Surname", "Doe") );
      modificationItems[1] = new ModificationItem(
        DirContext.ADD_ATTRIBUTE,
        new BasicAttribute("PhoneNumber", "111111111111") );

      context.modifyAttributes(entryDN,
        modificationItems);

      String filter = "(&(sn=Doe)(GivenName=*))";

      searchUsingFilterUsingSearch(filter);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }

  public void removeAttributes()
  {
    try{
      DirContext context = getInitialContext();
      ModificationItem modificationItems[] = new ModificationItem[1];

      modificationItems[0] = new ModificationItem(
        DirContext.REMOVE_ATTRIBUTE,
        new BasicAttribute("PhoneNumber") );

      context.modifyAttributes(entryDN,
        modificationItems);

      String filter = "(&(Surname=Doe)(GivenName=*))";

      searchUsingFilterUsingSearch(filter);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }





  public void getSelectedAttributesUsingGetAttributes()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter.
      // and has the attribute givenname
      String resultAttributes[] = {"GivenName", "Surname", "PhoneNumber"};

      // ignore name case
      // Search for objects with those matching attributes
      Attributes searchResults
        = context.getAttributes(entryDN,
        resultAttributes);
      printSearchResults(searchResults);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }


  public void getEveryAttributeUsingGetAttributes()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter.
      // and has the attribute givenname
      // ignore name case
      // Search for objects with those matching attributes
      Attributes searchResults
        = context.getAttributes(entryDN);
      printSearchResults(searchResults);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }

  public void getEveryAttributeUsingSearch()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter.
      // and has the attribute givenname
      BasicAttribute atr1 = new BasicAttribute("Description");
      BasicAttribute atr2 = new BasicAttribute("EMail Address");
      Attributes matchAttrs = new BasicAttributes(true);
      matchAttrs.put(atr1);
      matchAttrs.put(atr2);
      // ignore name case
      // Search for objects with those matching attributes
      NamingEnumeration searchResults
        = context.search(entryDN, matchAttrs);
      printSearchResults(searchResults);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }

  public void getSelectedAttributesUsingSearch()
  {
    try{
      DirContext context = getInitialContext();
      // search for the   object with attribute value uid is scarter.
      // and has the attribute givenname
      BasicAttribute atr1 = new BasicAttribute("Description");
      BasicAttribute atr2 = new BasicAttribute("EMail Address");
      Attributes matchAttrs = new BasicAttributes(true);
      matchAttrs.put(atr1);
      matchAttrs.put(atr2);

      String resultAttributes[] = {"GivenName", "Surname", "PhoneNumber",
        "Description","EMail Address"};;
      // ignore name case
      // Search for objects with those matching attributes
      NamingEnumeration searchResults
        = context.search(entryDN, matchAttrs,
        resultAttributes);
      printSearchResults(searchResults);

    }
    catch(NamingException namingException){
      namingException.printStackTrace();
    }
  }


   /**
   * Method
   *
   * Main Test Driver: java ejav.jndi.SimpleLookup
   * @param args
   *
   * @see
   */
  public static void main(String[] args)
  {

    if(args.length == 0){

    }
    // Set name of properties file to use in initializing the simple
    // lookup.
    String propertiesFileName = "jndi.Properties";
    //String propertiesFileName = "jndi.Properties";

    // Instantiate an instance of this NamingClient
    SampleSearchExampleNDS sampleSearchExampleNDS = null;

    try {
        sampleSearchExampleNDS =
          new SampleSearchExampleNDS(propertiesFileName);
    } catch (IOException ioException) {
      System.out.println("Error :" + ioException);
      System.exit(0);
    }


    sampleSearchExampleNDS.getEveryAttributeUsingSearch();
    sampleSearchExampleNDS.getSelectedAttributesUsingSearch();

    // Ask for objects with attribute surname is carter and which have
    // the "givenname" attribute.
    String filter = "(&(surname=Carter)(GivenName=*))" ;
    sampleSearchExampleNDS.searchUsingFilterUsingSearch(filter);
    sampleSearchExampleNDS.getEveryAttributeUsingGetAttributes();

  }



}
