package ejava.jndi.directory.dsml;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.Attribute;
import javax.naming.directory.SearchControls;
import java.util.Properties;
import java.io.IOException;
import java.io.FileInputStream;

/**
  * This example demonstrates main operations on a Directory Service
  *  Using DSML v2 via SOAP/HTTP
  */

public class SampleSOAPToDSMLServer
{
  DirContext context = null;
  Properties properties = new Properties();

  public SampleSOAPToDSMLServer(Properties properties)
  {
    super();
    this.properties = properties;
  }

  /**
   * Constructor
   * Naming Client which is initialized with a jndi.properties file.
   * Uncomment appropriate lines for each service provider in
   * jndi.properties.
   *
   * @param propertiesFileName
   */
  public SampleSOAPToDSMLServer(String propertiesFileName)
    throws IOException
  {
    // Open a file input stream for the associated properties
    // fileName.
    FileInputStream propertiesFileStream =
      new FileInputStream(propertiesFileName);
    // Now load the file input stream into the JNDI environment
    // properties.
    // This of course assumes that the file input stream is in a valid
    // Properties format.
    properties.load(propertiesFileStream);
  }


  /**
   * Method getInitialContext
   * @return DirContext
   */
  private DirContext getInitialContext()
    throws NamingException
  {
    // Create initial context if it is not already created
    if (context == null) {
      System.out.println("Properties :" + properties);
        // Read in a set of default paramters.
      context = new InitialDirContext(properties);
      System.out.println("Context is :" + context);
    }
    return context;
  }

  /**
   * Method searchUsingFilter
   * @param String filter
   * @return void
   */
  public NamingEnumeration searchUsingFilter(String filter)
  {
    NamingEnumeration searchResults = null;
    
    try{
      DirContext context = getInitialContext();
      SearchControls searchControls = new SearchControls();
      String[] resultAttributes = {"cn", "sn", "telephonenumber" };
      searchControls.setReturningAttributes(resultAttributes);
      // set the number of results
      searchControls.setCountLimit(10);

      // Search for objects with those matching attributes
      searchResults = context.search("ou=People ", 
                                     filter, searchControls);
    }
    catch(Exception ex){
        ex.printStackTrace();
    }
    
    return searchResults;
  }

  public static void main(String[] args)
    throws Exception
  {
    // get System properties
    Properties properties = System.getProperties();
    // Set name of properties file to use in initializing the simple
    // lookup.
    String propertiesFileName = "jndi.Properties";
    SampleSOAPToDSMLServer sampleSOAPToDSMLServer = null;
    try {
        // Instantiate an instance of this NamingClient
        sampleSOAPToDSMLServer =
          new SampleSOAPToDSMLServer(propertiesFileName);
    } catch (IOException ioException) {
      System.out.println("Error :" + ioException);
      System.exit(0);
    }
    // Ask for objects with attribute sn == Carter and which have
    // the "mail" attribute.
    String filter = "(&(sn=Carter)(mail=*))" ;
    NamingEnumeration searchResults 
        = sampleSOAPToDSMLServer.searchUsingFilter(filter);
        
    // Now print results
    System.out.println("DSML Search Results: \n");
    while (searchResults.hasMore()) {
        System.out.println("   >> " + searchResults.next());
    }
        
  }



}

