package ejava.jndi.directory.tradingexample;
import org.omg.CORBA.TCKind.*;
import org.omg.CORBA.*;
import org.omg.CosTrading.*;
import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Enumeration;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;


/**
  * This class instantiates CustomerImpl objects and
  * register them to the trading service.
  */
public class CustomerServer
{

  /**
  * This method checks whether the service type is available in
  * the trading service, if the type is not available it adds the type
  * to the TradingService.
  * we call the service type as CustomerObjectsService
  */
  public static void addServiceTypesToTradingService(
    Lookup lookup, ORB orb, String interfaceType,
    String serviceType)
  {

    try{
      ServiceTypeRepository serviceTypeRepository;
      serviceTypeRepository =
        ServiceTypeRepositoryHelper.narrow(lookup.type_repos());
      PropStruct[] serviceTypeProperties = new PropStruct[1];
      serviceTypeProperties[0] = new PropStruct();
      serviceTypeProperties[0].name = "customer_id";
      serviceTypeProperties[0].mode = PropertyMode.PROP_NORMAL;
      serviceTypeProperties[0].value_type =
          orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_long);
      // This type does not have any superTypes
      // But the add_type expects the superTypes as the
      // array for strings, so we need to send the superTypes as
      // 0 length array
      String[] superTypes = new String[0];
      serviceTypeRepository.add_type(serviceType,interfaceType,
          serviceTypeProperties, superTypes);
    }
    catch(Exception exception){
      System.out.println("Failed to add the service Types :"+ exception);
      exception.printStackTrace();
      //System.exit(0);
    }
  }
  /**
    * This method reads all the customers from the file and constructs
    * them and returns, it ignores first line in the file.
    * and assumes that there is entry for every filed in each line
    */
  public static Hashtable getAllCustomers()
  {
    String fileName = "CustomerInfo.txt";
    Hashtable returnValues = null;
    try{
      FileInputStream fin = new FileInputStream(fileName);
      InputStreamReader ins = new  InputStreamReader(fin);
      BufferedReader bf =
          new BufferedReader(ins);
      String currentLine = bf.readLine();
        // ignore firstline , which has headings

      while( (currentLine = bf.readLine()) != null){
        if(returnValues == null){
          returnValues = new Hashtable();
        }
        CustomerImpl presentCustomer = null;
        StringTokenizer st = new StringTokenizer(currentLine,",");
        while(st.hasMoreTokens()){
          String customerID = st.nextToken();
          String firstName = st.nextToken();
          String lastName = st.nextToken();
          String street = st.nextToken();
          String city = st.nextToken();
          String state = st.nextToken();
          String zip = st.nextToken();
          String email = st.nextToken();
          presentCustomer =
            new CustomerImpl(Integer.parseInt(customerID));
          presentCustomer.setFirstName(firstName);
          presentCustomer.setLastName(lastName);
          presentCustomer.setStreet(street);
          presentCustomer.setCity(city);
          presentCustomer.setState(state);
          presentCustomer.setZip(zip);
          presentCustomer.setEmail(email);
          returnValues.put(customerID, presentCustomer);
        }
      }
    }catch(Exception exception){
      System.out.println("Error Reading file "+exception);
      exception.printStackTrace();
    }


    return returnValues;

  }

  private static Customer getObject(ORB orb, CustomerImpl customerImpl)
    throws Exception
  {
     // get reference to rootpoa & activate the POAManager
     POA rootpoa =
             POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

     rootpoa.the_POAManager().activate();
     // get object reference from the servant
     org.omg.CORBA.Object reference =
             rootpoa.servant_to_reference(customerImpl);
     Customer customer = CustomerHelper.narrow(reference);

     return customer;

  }

  /**
    * gets customer obects from the file and registers
    * them with trading service using customer ID
    */
  public static void registerCustomerObjects(ORB orb,
    Register register, String serviceType)
    throws Exception
  {

    Hashtable customers = getAllCustomers();
    System.out.println("Custoemr :"+customers);
    if(customers != null){

      Enumeration keys = customers.keys();

      while(keys.hasMoreElements()){
        Property[] properties = new Property[1];
        String key = (String)keys.nextElement();
        Customer customer = getObject(orb, (CustomerImpl)customers.get(key));
        System.out.println("Registering :"+ customer +" ::::"+key);
        properties[0] = new Property();
        properties[0].name = "customer_id";
        Any any = orb.create_any();
        any.insert_long(customer.customer_id());
        properties[0].value = any;
        try{
          register.export(customer, serviceType, properties);
        }
        catch(Exception exception){
          System.out.println("Error Exporting :"+exception);
          exception.printStackTrace();
        }
      }
    }

  }

  public static void main(String[] args)
  {



    try{
      // initialize orb and get instance of it.
      org.omg.CORBA.ORB orb
         = org.omg.CORBA.ORB.init (args, System.getProperties());
      // get the TradingService Object
      org.omg.CORBA.Object tradingServiceReference =
        orb.resolve_initial_references("TradingService");
      Lookup lookup  = LookupHelper.narrow(tradingServiceReference);
      Register register = lookup.register_if();
      String interfaceType =
        "IDL:ejava/jndich20/tradingexample/Customer:1.0";
      String serviceType =   "CustomerObjectsService";
      // we need to add the service type to the trading service
      // we can add the objects to that service type.
      System.out.println("adding Service Type");
      addServiceTypesToTradingService( lookup,  orb, interfaceType,
        serviceType);
      System.out.println("Registering");
     // exporting the Customer instances to the TradingService
     registerCustomerObjects(orb,register,serviceType);
     // wait for the clients
     System.in.read();


    }
    catch(Exception exception){
      System.out.println("Error :" + exception);
      exception.printStackTrace();
      System.exit(0);
    }

  }



}