package ejava.jndi.naming;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.CompoundName;
import javax.naming.CompositeName;
import javax.naming.Binding;
import javax.naming.Name;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.File;

/**
 * This class implements the deletion and renaming of the objects in 
 * the context
 *
 */
public class SampleBindingExample extends SampleListAndSearchExample
{

  /**
   * Constructor
   *
   *
   * @param propertiesFile
   *
   * @see
   */
  public SampleBindingExample(String propertiesFile)
    throws IOException
  {
    super(propertiesFile);
  }

  /**
   * fsContext provider will not delete directory.
   * some of the providers may not support delete method at all.
   * LADP provider deletes corresponding name and object from directory
   * @param target to delete
   */
  public void deleteAnObject(String target)
    throws NamingException
  {
    Context context;

    context = super.getInitialContext();
    // unbinds the named object from the given namespace.
    context.unbind(target);

    // This method is called to check that it  removed the name.
    super.listNames("");
  }

  /**
   *This method deletes an object from naming service
   * @param target to delete
   */
  public void deleteAnObject(Name target)
    throws NamingException
  {
    Context context;

    context = super.getInitialContext();
    // unbinds the named object from the given namespace.
    context.unbind(target);

    // This method is called to check that it  removed the name.
    super.listNames("");
  }

  /**
   *This method renames any target ObjectName
   * @param target to rename
   */
  public void renameAnObject(String target, String newName)
    throws NamingException
  {
    Context context;

    context = super.getInitialContext();

    context.rename(target, newName);

    // This method is called to check that it removed the name.
    super.listNames("");
  }

  /**
   *This method renames any target ObjectName
   * @param target to rename
   */
  public void renameAnObject(Name target,  Name newName)
    throws NamingException
  {
    Context context;

    context = super.getInitialContext();

    context.rename(target, newName);
  }

  /**
   * bind a new Object for the existing name
   * @param target to bind
   */
  public void bindNewObject(String target, Object newObject)
    throws NamingException
  {
    Context context;

    context = super.getInitialContext();

    context.rebind(target, newObject);
  }

  /**
   * bind a new Object for the existing Name
   * @param target to bind
   */
  public void bindNewObject(Name target, Object newObject)
    throws NamingException
  {
    Context context;

    context = super.getInitialContext();

    context.rebind(target, newObject);
  }

  /**
   *main method to test the SampleBindingExample class
   */
  public static void main(String args[])
  {

    // Ensure that a name to lookup is passed as a parameter.
    if (args.length != 1) {
      System.out.println("Run example as: "
         + "java ejava.jndi.naming.SampleBindingExample "
         + "<targetObject>");
      System.exit(0);
    }

    // Set name of properties file to use in initializing the lookup.
    String propertiesFileName = "jndi.Properties";
    String target = args[0];

    try {
      SampleBindingExample simpleBindingExample =
        new SampleBindingExample(propertiesFileName);

      // to test the delete object construct CompoundName like in
      // SimpleListAndSearchExample and
      // call deleteAnObject.
      String newObjectName = "newObject1";
      Object obj = simpleBindingExample.lookupBoundObject(target);

      // Renaming can be done only for the object not the subcontext.
      // file system Service provider does not know how to rename the 
      // directory. It knows how to rename file only.
      simpleBindingExample.renameAnObject(target, newObjectName);
      simpleBindingExample.bindNewObject(target, obj);

      // To delete an object always give a complete
      //    Name or Composite Name
      simpleBindingExample.deleteAnObject(newObjectName);
    } catch (IOException ioException) {
      System.out.println("Error opening jndi.properties file :"
         + ioException);
      System.exit(0);
    } catch (NamingException namingException) {
      System.out.println("Error  :" + namingException);
      System.exit(0);
    }
  }
}


