package ejava.jndi.naming;

import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;
import org.omg.CORBA.ORB;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import javax.naming.NamingException;
import javax.naming.Context;
import javax.naming.InitialContext;
import ejava.jndi.naming.sample.MessagePOA;
import ejava.jndi.naming.sample.MessageHelper;
import ejava.jndi.naming.sample.Message;

/**
 * CosNaming Server class implementation
 */
public class SampleCosNamingServer extends MessagePOA
{

  /**
   * Constructor
   */
  public SampleCosNamingServer()
  {
    super();
  }

  /**
   * Implementation of Sample Interface only method
   * @param s input string
   * @return string to return after manipulation
   */
  public String sayHi(String s)
  {
    return "You Sent :" + s;
  }

  /**
   * This method binds the object with different names
   * @param context to get names to bind
   */
  public static void bindWithDifferentNames(Context context,
   Message message)
   throws NamingException
  {
    Properties prop = (Properties) context.getEnvironment();
    String namingServerURL =
      (String) prop.get("java.naming.provider.url");

    String simpleName1 = (String) prop.get("SIMPLE_NAME1");
    String simpleName2 = (String) prop.get("SIMPLE_NAME2");
    // binding this object with different Names
    context.bind(namingServerURL + simpleName1, message);
    System.out.println("Bound As :" + namingServerURL + simpleName1);
    context.bind(namingServerURL + simpleName2, message);
    System.out.println("Bound As :" + namingServerURL + simpleName2);
  }

  /**
   * main method to test the functionality of the server
   */
  public static void main(String[] args) {
    if (args.length == 0) {
      System.out.println(" You should provide properties file :" +
          "java ejava.jndi.naming.SampleCosNamingServer jndi.properties");
      System.exit(0);
    }

    String propertiesFileName = args[0];

    try {
      FileInputStream fin = new FileInputStream(propertiesFileName);
      Properties properties = new Properties();
      properties.load(fin);

      ORB    orb  = ORB.init(args, null);

      // get reference to rootpoa & activate the POAManager
      POA rootpoa =
        POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

      rootpoa.the_POAManager().activate();

      // createQueryServerImplementation
      SampleCosNamingServer serverObjectImpl =
        new SampleCosNamingServer();

      // get object reference from the servant
      org.omg.CORBA.Object reference =
        rootpoa.servant_to_reference(serverObjectImpl);
      Message message = MessageHelper.narrow(reference);

      Context context = new InitialContext(properties);
      serverObjectImpl.bindWithDifferentNames(context,message);
      System.out.println("Bound : Waiting for clients...");

      orb.run();
    } catch (NamingException namingException) {
      System.out.println(" Failed to Bind :" + namingException);
      namingException.printStackTrace();
      System.exit(0);
    } catch (IOException ioException) {
      System.out.println(" Failed to open properties file :"
                         + ioException);
      System.exit(0);
    }
    catch (Exception exception) {
      exception.printStackTrace();
      System.exit(0);
    }
  }
}


