package ejava.jndi.naming;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Hashtable;
import java.util.Properties;
import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.io.FileInputStream;
import java.io.IOException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.rmi.RMISecurityManager;

/**
 * JRMP server using naming context class implementation
 */
public class SampleJRMPServerUsingNamingContext
  extends UnicastRemoteObject implements Sample
{

  /**
   * Constructor
   *
   */
  public SampleJRMPServerUsingNamingContext()
    throws RemoteException
  {
    super();
  }

  /**
   * Implementation of Sample Interface only method
   * @param s input string
   * @return string to return after manipulation
   */
  public String sayHi(String s)
    throws RemoteException
  {
    return "You Sent:" + s;
  }

  /**
   * This method binds the object with different names
   * @param context to get names to bind
   */
  public void bindWithDifferentNames(Context context)
   throws   NamingException,
            MalformedURLException,
            AlreadyBoundException
  {
    Properties properties = (Properties) context.getEnvironment();
    String namingServerURL =
      (String) properties.get("java.naming.provider.url");
    String simpleName1 = (String) properties.get("SIMPLE_NAME1");
    String simpleName2 = (String) properties.get("SIMPLE_NAME2");

    // binding this object with different Names
    context.bind(namingServerURL + simpleName1, this);
    System.out.println("Bound As :" + namingServerURL + simpleName1);
    context.bind(namingServerURL + simpleName2, this);
    System.out.println("Bound As :" + namingServerURL + simpleName2);

    String compoundName1 = (String) properties.get("COMPOUND_NAME1");
    String compoundName2 = (String) properties.get("COMPOUND_NAME2");

    context.bind(compoundName1, this);
    System.out.println("Bound as :" + compoundName1);
    context.bind(compoundName2, this);
    System.out.println("Bound as :" + compoundName2);
  }

  /**
   * main method to test the SampleJRMPServerUsingNamingContext
   * class functionality
   */
  public static void main(String[] args)
  {
    if (args.length == 0) {
      System.out.println("Usage : java -Djava.security.policy"
      + "=rmi.policy ejava.jndi.naming.SampleJRMPServerUsingNamingContext"
      + " jndich20rmiContext.Properties");
      System.exit(0);
    }

    String propertiesFileName = args[0];

    try {
      FileInputStream fin = new FileInputStream(propertiesFileName);
      Properties properties = new Properties();

      properties.load(fin);

      if (System.getSecurityManager() == null) {
        System.setSecurityManager(new RMISecurityManager());
      }

      SampleJRMPServerUsingNamingContext serverObject =
        new SampleJRMPServerUsingNamingContext();
      Context namingContext = new InitialContext(properties);

      serverObject.bindWithDifferentNames(namingContext);
      System.out.println("Bound : Waiting for clients...");
    }
    catch (IOException ioException) {
      System.out.println("Failed to open properties :" + ioException);
      System.exit(0);
    }
    catch (AlreadyBoundException alreadyBoundException) {
      System.out.println("Already Bound :" + alreadyBoundException);
      System.exit(0);
    }
    catch (NamingException namingException) {
      System.out.println(" Naming Error: " + namingException);
      namingException.printStackTrace();
      System.exit(0);
    }
  }
}


