package ejava.jsp.tagext;

import java.util.Iterator;
import java.util.HashMap;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.DynamicAttributes;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.tagext.TryCatchFinally;

/**
 *  This tag just demonstrates the use of the DynamicAttributes
 *  interface, as well as TryCatchFinally.  When the tag is
 *  invoked, it just shows the names and values of the attributes
 */
public class DynAttribExampleTag extends TagSupport
            implements DynamicAttributes, TryCatchFinally
{

    private HashMap attributeMap;

    public DynAttribExampleTag()
    {
        attributeMap = new HashMap();
    }

    /**
     * implementation of DynamicAttributes interface
     */
    public void setDynamicAttribute(String uri, 
                                    String localName, 
                                    Object value)
    {
        attributeMap.put(localName, value);
    }

    /**
     * implementation of TryCatchFinally
     */
    public void doCatch(Throwable t) throws Throwable
    {
        System.out.println(t.toString());
    }

    public void doFinally()
    {
        // perform any cleanup necessary,
        // such as open JDBC connections, file handles, etc.
        // is always called when tag exits
        System.out.println("doFinally called");
    }

    /**
     *  Called by the container when the start tag is processed
     */
    public int doStartTag() throws JspException
    {
        Iterator it = attributeMap.keySet().iterator();

        while (it.hasNext())
        {
            String name = (String)it.next();
            String value = (String)attributeMap.get(name);
            try
            {
                pageContext.getOut().println("Attribute name = " 
                                             + name + "<br>");
                pageContext.getOut().println("Attribute value = " 
                                             + value + "<br>");
            }
            catch (Exception e)
            {
                throw new JspException(e);
            }
        }

        return TagSupport.SKIP_BODY;
    }
}
