package ejava.rmi.queryservicejrmp;

import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.io.IOException;
import java.util.Vector;
import java.util.prefs.Preferences;
import ejava.rmi.ClientReceiver;
import ejava.rmi.Customer;
import ejava.rmi.QueryServer;
import ejava.rmi.Order;

public class QueryClient
{

  public void list(String rmiURL)
  {
    try {
      String[] names = Naming.list(rmiURL);

      for (int i = 0; i < names.length; i++) {
        System.out.println("Registered objects :" + names[i]);
      }
    } catch (Exception e) {
      System.out.println("Failed to lookup :" + e);
    }
  }

  public static void main(String[] args)
  {
    Preferences preferences =
      Preferences.userNodeForPackage(
         ejava.rmi.QueryServer.class);

    try {
      // Get query server name
      String queryServerName =
        preferences.get("QUERY_SERVER_NAME",null);

      // Get registry name
      String registryURL =
        preferences.get("REGISTRY_SERVER_URL",null);
      String name =  registryURL + queryServerName;
      System.out.println("QueryServerName :" + name);
      
      // Set security manager
      if (System.getSecurityManager() == null) {
        System.setSecurityManager(new RMISecurityManager());
      }
      System.out.println("Client Looking for Server :" + name);
      System.out.println("Type Enter Key");
      System.in.read();    // to hold the screen

      // Lookup OrderManager reference
      QueryServer qserver = (QueryServer) Naming.lookup(name);
      System.out.println("Client querying for customers :");
      System.out.println("Type Enter Key");
      System.in.read();

      // Query for customers and traverse results
      Vector v = qserver.findCustomersWhoOrderedForMoreThan(10.0f);

      System.out.println("Resultant Customers: " + v);

      for (int i = 0; i < v.size(); i++) {
        String cid = (String) v.elementAt(i);
        Customer c = (Customer) Naming.lookup(registryURL+cid);

        System.out.println(i + " : is " + c);
        System.out.println("FirstName :" + c.getFirstName());
        System.out.println("LastName:" + c.getLastName());
      }

      System.out.println("Server Writing Orders ");
      System.out.println("Type Enter Key");

      // Register client callback with query server
      ClientReceiverImplementation cr =
        new ClientReceiverImplementation();
      qserver.register("ClientReceiver", cr);

      // Induce callbacks to be made by requesting order info
      qserver.findOrdersWhichAreFrom("VA");
      QueryClient qc = new QueryClient();

      System.out.println("Now list all registered objects :");
      System.out.println("Type Enter Key");
      System.in.read();
      qc.list(registryURL);

    } catch (RemoteException remoteEx) {
      System.out.println("RemoteException: " +
         remoteEx.getMessage());
      remoteEx.printStackTrace();
    } catch (MalformedURLException malformedURLException) {
      System.out.println("RemoteException: " + malformedURLException);
      malformedURLException.printStackTrace();
    } catch (NotBoundException notBoundEx) {
      System.out.println("NotBoundException: " + notBoundEx);
      notBoundEx.printStackTrace();
    } catch(IOException ioException){
      System.out.println("IOException: " + ioException);
      ioException.printStackTrace();
    }
  }

}

