package ejava.servlets;

// 
import java.io.IOException;
import java.io.PrintWriter;
import java.util.StringTokenizer;
import java.util.Vector;

// 
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

public class BrowseShirtsServlet extends HttpServlet
{
   public static final String SERVLET_NAME = "/BrowseShirtsServlet";
   public static final String BROWSED_SHIRTS = "BROWSED_SHIRTS";
   private final String SHIRT_TYPE = "TYPE";
   private final String SHIRT_SIZE = "SIZE";
   private final String SHIRT_COLOR = "COLOR";

/**
  * FCalled when a GET request sent to this servlet
  */
  public void doGet (HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException, IOException
  {
    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Write the body of the page
    writePageBody(printWriter, servletRequest);
    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }


  /**
  * Write Page Body
  */
  private void writePageBody(PrintWriter printWriter,
    HttpServletRequest servletRequest) 
  {
    // Write the top half of page
    ServletsHelper.writePageTopArea(printWriter);
    // Divide rest of page into two table columns
    printWriter.println("<TABLE BORDER=0 COLS=2 WIDTH=\"700\" >");
    printWriter.println(" <TR> ");
    String serverInfo = ServletsHelper.getServerInfo(servletRequest);
    // Write left hand side (buttons)
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Write the center area for About Us servlet
    writePageCenterAreaForForm( servletRequest, printWriter);
    // close the TABLE
    printWriter.println("</TR>");
    printWriter.println("</TABLE> <br>&nbsp;");

  }

/**
  * This method called when POST request is made
  */
  public void doPost(HttpServletRequest servletRequest, HttpServletResponse servletResponse)
    throws ServletException, IOException
  {
    // Get request information first
    String queriedSize = servletRequest.getParameter(SHIRT_SIZE);
    String queriedColor = servletRequest.getParameter(SHIRT_COLOR);
    String serverInfo =  ServletsHelper.getServerInfo(servletRequest);

    // Get Session
    HttpSession session = servletRequest.getSession();
    if(session == null){
      session = servletRequest.getSession(true);
    }

    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);

    ServletsHelper.writePageTopArea(printWriter);
    // Divide the remaining page into two Table columns.
    printWriter.println("<TABLE BORDER=0 COLS=2 WIDTH=\"700\" >");
    printWriter.println(" <TR> ");
    // Write left hand side (buttons)
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Write center page area
    writePageCenterAreaResults(printWriter, queriedSize, queriedColor,
                               serverInfo, session);
    // Close the TABLE
    printWriter.println("</TR>");
    printWriter.println("</TABLE> <br>&nbsp;");
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }

/**
  * Write out the browse form in the center of the page
  */
  private void writePageCenterAreaForForm(HttpServletRequest servletRequest,
    PrintWriter printWriter)
  {
    String postValue = "http://"
        + ServletsHelper.getServerInfo(servletRequest) + SERVLET_NAME;

    printWriter.println("<TD VALIGN=TOP >");
    printWriter.println("<FORM METHOD =\"POST\" NAME=\"queryForm\" " +
       " ACTION=\""+postValue+"\"> ");
    printWriter.println("<TABLE  bgcolor=\"#CCCC99\" "+
      "cellpadding=\"3\" cellspacing=\"0\" border=\"1\"> ");
    printWriter.println(" <TR> <TD> Shirt Size : </TD> "+
      " <TD colspan=\"1\"> <SELECT NAME=\"" +SHIRT_SIZE+"\" >  " +
      " <OPTION value=\"XL\" selected >Extra Large " +
      "  <OPTION value=\"L\" > Large  " +
      "  <OPTION value=\"M\" > Medium " +
      "  <OPTION value=\"S\" > Short " +
      " </SELECT><br></TD> </TR> " );
    printWriter.println("</TD>");
    printWriter.println(" <TR> <TD> Shirt Color : </TD> "+
      " <TD colspan=\"1\"> <SELECT NAME=\""+SHIRT_COLOR+"\" >  " +
      " <OPTION value=\"White\" selected >White " +
      "  <OPTION value=\"Red\" > Red  " +
      "  <OPTION value=\"Green\" > Green " +
      "  <OPTION value=\"Gray\" > Yellow " +
      " </SELECT><br></TD> </TR> " );
    printWriter.println("<TR> <TD colspan=\"2\">  " +
      " <INPUT type=SUBMIT Name = \"query\" Value=\"query\"> " +
      " </TD> </TR> ");
    printWriter.println("</TABLE> </FORM> ");
  }

/**
  * Write out the results from the browse query
  */
  private  void   writePageCenterAreaResults(PrintWriter printWriter,
    String queriedSize, String queriedColor, String serverInfo,
    HttpSession session)
  {
    serverInfo = "http://"+serverInfo+ CartServlet.SERVLET_NAME;;
    printWriter.println("<TD VALIGN=TOP>");
    printWriter.println("<TABLE  bgcolor=\"#CCCC99\" "+
      "cellpadding=\"1\" cellspacing=\"0\" border=\"1\"> ");
    Vector browsedShirts = getQueriedShirts( queriedSize, queriedColor);

    System.out.println("session "+session);
    session.setAttribute(BROWSED_SHIRTS, browsedShirts);

    if(browsedShirts.size() == 0){
     printWriter.println("<TR> <TD><STRONG> NO Shirts Match Your Query Criteria"+
      " </STRONG> </TD> </TR>");
    }
    printWriter.println("<TR> <TD COLSPAN=2> Queried For :</TD> </TR> ");
    printWriter.println("<TR> <TD> "+SHIRT_SIZE +": </TD> <TD>  "+
      queriedSize+"</TD> </TR> ");
    printWriter.println("<TR> <TD> "+SHIRT_COLOR +": </TD> <TD>  "+
      queriedColor+"</TD> </TR> ");
    printWriter.println("<TR> </TR>");
    printWriter.println("<TR> </TR>");
    printWriter.println("<TR> </TR>");
    for(int i = 0; i< browsedShirts.size(); i++){
      TShirt tShirt = (TShirt)browsedShirts.elementAt(i);
      printWriter.println("<TR> <TD VALIGN=TOP COLSPAN=2> "+
        tShirt.getTShirtAsHTMLTableForBrowing(serverInfo)+
        "</TD></TR>");
    }
    printWriter.println("</TABLE>");
    printWriter.println("</TD>");
  }

/**
  * For each TShirt query size and color, find Vector of TShirts
  */
  private Vector getQueriedShirts(String queriedSize,
    String queriedColor)
  {
    String nShirtsString = getInitParameter("numberOfShirts");
    Vector returnValue = new Vector();
    int nShirts = Integer.parseInt(nShirtsString);
    for(int i = 0; i< nShirts; i++){
      String parameterName = "shirt_"+i;
      String unparsedString =
        getInitParameter(parameterName);
      TShirt tShirt = constructAShirt(unparsedString);
      if(tShirt != null &&
         tShirt.getSize().trim().equals(queriedSize.trim()) &&
         tShirt.getColor().trim().equals(queriedColor.trim()) ){
        returnValue.addElement(tShirt);
      }
    }
     return returnValue;
  }

/**
  * For this String of data, parse it, and construct a TShirt object
  */
  private TShirt constructAShirt(String unParsedString)
  {
    StringTokenizer st = new StringTokenizer(unParsedString,",");
    while(st.hasMoreElements()){
      int shirtID = Integer.parseInt(st.nextToken());
      String size = st.nextToken();
      String color = st.nextToken();
      double priceForItem = (Double.parseDouble((String)st.nextElement()));
      String designFront = st.nextToken();
      String designBack = st.nextToken();
      String frontPicture = st.nextToken();
      String backPicture = frontPicture;
      TShirt tShirt =
        new TShirt(shirtID,size,color,designFront,designBack,
          frontPicture,backPicture, priceForItem);
      return tShirt;
    }
    return null;
  }
}

