package ejava.servlets;

// 
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

// 
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
  * This Servlet keeps track of shopping cart entries added to it.
  */
public class CartServlet extends HttpServlet
{
  public static final String SERVLET_NAME="/CartServlet";

/**
  * This method called when GET request received
  */
 public void doGet(HttpServletRequest servletRequest,
  HttpServletResponse servletResponse)
  throws ServletException, IOException
 {
  // Check if there is a session
   HttpSession session = servletRequest.getSession();
   ShoppingCart cart =
    (ShoppingCart)session.getAttribute(ShoppingCart.SHOPPING_CART_OBJECT);
   // If no cart session, then display empty information.
   if(cart == null){
     displayNoItemsInTheCart( servletRequest, servletResponse);
   }
   else{
    // Else display the cart contents
    displayItemsInTheCart(servletRequest, servletResponse, cart);
   }

 }

/**
  * Handles POST request sent to Servlet
  */
  public void doPost(HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException, IOException
 {
   log("posted to :"+SERVLET_NAME);

   // Get session information 
   HttpSession session = servletRequest.getSession(false);
   // Retrieve shopping cart from session
   ShoppingCart cart =
    (ShoppingCart)session.getAttribute(ShoppingCart.SHOPPING_CART_OBJECT);
   // If no cart, then make a new one
   if(cart == null){
     cart = new ShoppingCart();
   }

   // Get shirt ID from POST
   String selectedShirtIDString =
    servletRequest.getParameterValues(TShirt.SHIRT_ID)[0];
   // Get number of shirts from POST
   String numberSelectedString =
    servletRequest.getParameterValues(TShirt.ORDERED_FIELD)[0];

   // Parse shirt ID and number of shirts
   int shirtID = Integer.parseInt(selectedShirtIDString);
   int numberSelected = Integer.parseInt(numberSelectedString);

   // Get list of shirts that were temporarily constructed in session
   Vector browsedShirts
    = (Vector)session.getAttribute(BrowseShirtsServlet.BROWSED_SHIRTS);
   
   // Loop through each shirt in this list
   for(int i = 0; i<browsedShirts.size(); i++){
     // Cast the element to a TShirt object
     TShirt tShirt = (TShirt)browsedShirts.elementAt(i);
     // If the TShirt object's ID equals the received shirt ID... 
     if(tShirt.getItemID() == shirtID){
        // set quentity selected
        tShirt.setQuantity(numberSelected);
        // set totalPrice
        tShirt.setTotalPrice(tShirt.getUnitPrice()*numberSelected);
       // add item to the Cart
       cart.addItem(tShirt);
     }
   }

   // Add shopping cart to session
   session.setAttribute(ShoppingCart.SHOPPING_CART_OBJECT, cart);
   // Display info
   displayAddedOrFailedToAdd(servletRequest, servletResponse);
 }
 
/**
  * Display empty cart information
  */
 private void displayNoItemsInTheCart( HttpServletRequest servletRequest,
  HttpServletResponse servletResponse)
  throws ServletException, IOException
 {
     // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Write top half and left hand side of window
    ServletsHelper.writePageTopArea(printWriter);
    String serverInfo = ServletsHelper.getServerInfo(servletRequest);
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Write empty cart inforamtion to center of page
    printWriter.println("<TD  VALIGN = TOP> ");
    printWriter.println("  The Cart is Empty, Go to <STRONG> Browse </STRONG>" +
       " Area to browse shirts and add new Shirts to the Cart <BR> </TD> ");
    printWriter.println(" </TD> </TR> </TABLE> ");

    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
 }

/**
  * Display items in the cart to the user
  */
  private void displayItemsInTheCart(HttpServletRequest servletRequest,
    HttpServletResponse servletResponse,ShoppingCart shoppingCart)
    throws ServletException, IOException
  {
    // Create initial document type and headers
    PrintWriter printWriter =
      ServletsHelper.createInitialDocumentResponse(servletResponse);
    // Write top half and left hand side of page
    ServletsHelper.writePageTopArea(printWriter);
    String serverInfo = ServletsHelper.getServerInfo(servletRequest);
    ServletsHelper.writePageLeftArea(printWriter, serverInfo);
    // Show cart contents in the center of page
    printWriter.println("<TD VALIGN=TOP>" );
    printWriter.println(shoppingCart.getCartAsHTMLTable());
    printWriter.println(" </TD> ");

    printWriter.println("</TR> </TABLE> ");

    // Close with BODY and HTML closing tags
    ServletsHelper.createClosingDocumentResponse(printWriter);
  }

/**
  * Display fact that item was added to the cart
  */
   private void displayAddedOrFailedToAdd(HttpServletRequest servletRequest,
    HttpServletResponse servletResponse)
    throws ServletException , IOException
   {
     // Create initial document type and headers
     PrintWriter printWriter =
        ServletsHelper.createInitialDocumentResponse(servletResponse);
     // Display top half and left hand side of page
     ServletsHelper.writePageTopArea(printWriter);
     String serverInfo = ServletsHelper.getServerInfo(servletRequest);
     ServletsHelper.writePageLeftArea(printWriter, serverInfo);
     // Indicate that item was added to the cart
     printWriter.println("<TD  VALIGN = TOP> ");
     printWriter.println("  <TD> Selected Shirt is Added to Cart, <STRONG> Browse </STRONG> "+
      " to add more shirts to cart </TD> ");
     printWriter.println(" </TD> </TR> </TABLE> ");

     // Close with BODY and HTML closing tags
     ServletsHelper.createClosingDocumentResponse(printWriter);
   }
}
