package ejava.servlets;

import java.io.IOException;


import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 *  Finds a Customer object in the HttpSession.
 *  Prints request string along with customer name
 *  out to a log file, to audit customer trail 
 *  through the site.
 */
public class CustomerAuditFilter implements Filter
{

    private FilterConfig filterConfig = null;

    public CustomerAuditFilter()
    {
    }

    /**
     * Initialize the filter
     *
     * @param filterConfig The filter configuration object
     */
    public void init(FilterConfig filterConfig) throws ServletException
    {
	this.filterConfig = filterConfig;
    }

    /**
     *  Clean up when filter exits
     */
    public void destroy() 
    {
        filterConfig = null;
    }


    /**
     * The container invokes this method for every filter in the
     * chain, in the order they were specified in the deployment
     * descriptor.
     * The filter has the choice of returning a response here, thus
     * ending the chain, or else it must call 
     * chain.doFilter(request, response) to continue processing.
     */
    public void doFilter(ServletRequest request, 
                         ServletResponse response,
                         FilterChain chain)
	throws IOException, ServletException 
    {
        if (filterConfig == null)
	    return;

	// Render the HTTP servlet request properties
	if (request instanceof HttpServletRequest) 
        {
	    HttpServletRequest httpReq = (HttpServletRequest)request;
            HttpSession sess = httpReq.getSession();
            Customer customer = 
		(Customer)sess.getAttribute(Customer.CUSTOMER_OBJECT);
            if (customer != null)
		logCustomerRequest(customer, httpReq);
	}

	// pass the request on
	chain.doFilter(request, response);
    }


    /**
     * Outputs message to the log file.
     */
    private void logCustomerRequest(Customer customer, 
                                    HttpServletRequest request)
    {
	ServletContext ctx = filterConfig.getServletContext();

	String requestURI = request.getRequestURI();
	String customerName = customer.getFirstName() + " " + 
                              customer.getLastName();

	ctx.log("User " + customerName + " requested " + requestURI);
    }
}

				    
