package ejava.servlets;

//
import java.io.Serializable;
import java.text.DateFormat;
import java.util.Date;

/**
  * Encapsulates Order information for a Customer
  */
public class Order implements Serializable
{
  public static final String ORDER_ID = "orderID";
  public static final String ORDER_DATE = "orderDate";
  public static final String SHIP_INSTRUCTION = "shipInstruct";
  public static final String SHIP_DATE = "shipDate";
  public static final String SHIP_WEIGHT="shipWeight";
  public static final String PAID_DATE = "paidDate";
  public static final String ORDER_OBJECT = "ORDER_OBJECT";

  private int orderID;
  private Date orderDate;
  private String shipInstruct;
  private Date   shipDate;
  private double shipWeight;
  private Date paidDate;
  private double shipCharge;
  private Address billingAddress;
  private Address shippingAddress;
  private TShirt[] items;
  private Customer customer;


  public Order(int orderID,
    Date orderDate, String shipInstruct,
    Date shipDate, double shipWeight,
    Date paidDate, Address billingAddress,
    Address shippingAddress , TShirt[] items )
  {
    this.orderID = orderID;
    this.orderDate = orderDate;
    this.shipInstruct = shipInstruct;
    this.shipDate = shipDate;
    this.shipWeight = shipWeight;
    this.paidDate = paidDate;
    this.billingAddress = billingAddress;
    this.shippingAddress = shippingAddress;
    this.items = items;
  }


  public Order()
  {
    // default Constructor
  }


/**
  * Setters
  */
  public void setOrderID(int newOrderID)
  {
    orderID = newOrderID;
  }
  public void setOrderDate(Date newOrderDate)
  {
    orderDate = newOrderDate;
  }
  public void setShippingInstructions(String shipInstructions)
  {
    shipInstruct = shipInstructions;
  }
  public void setShipDate(Date newShipDate)
  {
    shipDate = newShipDate;
  }
  public void setShipWeight(double newShipWeight)
  {
    shipWeight = shipWeight;
  }
  public void setPaidDate(Date newPaidDate)
  {
    paidDate = newPaidDate;
  }
  public void setBillingAddress(Address newBillingAddress)
  {
    billingAddress = newBillingAddress;
  }
  public void setShippingAddress(Address newShippingAddress)
  {
    shippingAddress = newShippingAddress;
  }
  public void setItems(TShirt[] newItems)
  {
    items =  newItems;
  }
  public void setCustomer(Customer newCustomer)
  {
    customer = newCustomer;
  }
  public void setShipCharge(double newShipCharge)
  {
    shipCharge = newShipCharge;
  }

/**
  * Getters
  */
  public TShirt[] getItems()
  {
    return items;
  }
  public Address getShippingAddress()
  {
    return shippingAddress;
  }
  public Address getBillingAddress()
  {
    return billingAddress;
  }
  public Date getPaidDate()
  {
    return paidDate;
  }
  public double getShipWeight()
  {
    return shipWeight;
  }
  public Date getOrderDate()
  {
    return orderDate ;
  }
  public String  getShippingInstructions()
  {
    return shipInstruct ;
  }
  public Date getShipDate()
  {
    return shipDate ;
  }
  public int getOrderID()
  {
    return orderID;
  }

  public Customer getCustomer()
  {
    return customer;
  }
  public double getShipCharge()
  {
      return shipCharge;
  }

/**
  * Get as String
  */
  public String toString()
  {
    return "Order ID :" +orderID +" Order Date :" + orderDate +
      " Shiping Instructions :"+shipInstruct +" Ship Date :"+shipDate +
      " Ship Weight :" +shipWeight +" Paid Date :"+paidDate +
      " Billing Address :" +billingAddress.toString() +
      " Shipping Address :" +shippingAddress.toString()+
       " items :" +items + " Customer :" + customer.toString();

  }

  public Object clone()
  {
    return new Order(orderID, orderDate, shipInstruct, shipDate,
      shipWeight, paidDate , billingAddress, shippingAddress,
      items);
  }

/**
  * Get Order as a formatted HTML table
  */
  public String getOrderAsHTMLTable()
  {
    StringBuffer outputString = new StringBuffer();
    outputString.append("<TABLE BORDER=0"+
      " CELLSPACING=0 CELLPADDING=0  BGCOLOR=\"#FFCC66\" >");
    outputString.append("<TR> <TD> <STRONG> Order Information <STRONG> </TD> </TR>");
    outputString.append(" <TR> <TD > "+ ORDER_ID +
      " :  </TD> <TD> " +orderID +" <TD> </TR> ");
    outputString.append(" <TR> <TD> "+ ORDER_DATE +
      " :  </TD> <TD> " +DateFormat.getInstance().format(orderDate) +" <TD> </TR> ");
    outputString.append(" <TR> <TD> "+ PAID_DATE +
      " :  </TD> <TD> " +DateFormat.getInstance().format(paidDate) +" <TD> </TR> ");
    outputString.append(" <TR> <TD> "+ SHIP_INSTRUCTION +
      " :  </TD> <TD> " +shipInstruct +" <TD> </TR> ");
    outputString.append(" <TR> <TD> "+ SHIP_DATE +
      " :  </TD> <TD> " +DateFormat.getInstance().format(shipDate) +" <TD> </TR> ");
    outputString.append(" <TR> <TD> "+ SHIP_WEIGHT +
      " :  </TD> <TD> " + shipWeight +" <TD> </TR> ");
    if(billingAddress != null){
      outputString.append("<TR> <TD VALIGN=TOP >" );
      // add Billing Address
      outputString.append(billingAddress.getAddressAsHTMLTable("Billing Address "));
      outputString.append("</TD></TR>");
    }
    if(shippingAddress != null){
      // add Shipping Address
      outputString.append("<TR> <TD VALIGN=TOP >" );
      // add Billing Address
      outputString.append(shippingAddress.getAddressAsHTMLTable("Shipping Address "));
      outputString.append("</TD></TR>");
    }
    if(items != null){
      // display All Items Information
      for(int i = 0; i< items.length;i++){
        TShirt tshirt = items[i];
        outputString.append("<TR> <TD VALIGN=TOP >" );
        outputString.append(tshirt.getTShirtAsHTMLTable());
        outputString.append("</TD></TR>");
      }
    }
    outputString.append("</TABLE>");
    return outputString.toString();
  }
}
