package ejava.tcpip;

import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.io.IOException;
import java.util.prefs.Preferences;
import java.io.FileInputStream;
import java.io.ObjectOutputStream;

/**
 * This class opens sever socket and starts listening on that
 * server
 */
public class Server {
    private int            defaultPort = 9000;
    private ServerSocket   serverSocket;
    private Socket         newConnection;
    private DatabaseHelper dbHelper;
    private int            numberOfClients = 10;
    private int            connectedClientsCounts = 0;

    public Server(int serverPort) {
        if (serverPort > -1) {
            this.defaultPort = serverPort;
        } 
    }

    public void startServer() throws IOException, Exception {
        Preferences preferences = 
            Preferences.userNodeForPackage(Server.class);

        // Get number of clients if there isn't any in the config.xml
        // just get 1
        int         numberOfClients = 
            preferences.getInt("NUMBER_OF_CLIENTS", 1);

        serverSocket = new ServerSocket(defaultPort, numberOfClients);

        // Lazy instantiation of database helper and create connection
        if (dbHelper == null) {
            dbHelper = new DatabaseHelper();

            dbHelper.makeDbConnection();
        } 

        /**
         * until user exists from the server using keyboard, CTRL-C
         * or by shutting down from the server, it waits for
         * clients to connect
         */
        while (true) {
            newConnection = serverSocket.accept();

            ServerHelper sHelper = new ServerHelper(newConnection, 
                                                    dbHelper);

            sHelper.start();
        } 
    } 

    public void cleanResources() throws IOException {
        serverSocket.close();
    } 

    public void finalize() throws Throwable {
        try {
            cleanResources();
        } catch (IOException e) {}

        super.finalize();
    } 

    public static void main(String[] args) {
        Preferences preferences = 
            Preferences.userNodeForPackage(ejava.tcpip.Server.class);

        // Get and parse server port
        int serverPort = preferences.getInt("SERVER_PORT", -1);

        if (serverPort == -1) {
            System.out.println("There should be a valid SERVER_PORT");
            System.exit(0);
        } 

        try {

            // Create new instance of this server and start it
            Server s = new Server(serverPort);

            s.startServer();
        } catch (Exception e) {
            System.out.println("Error: " + e);
            System.exit(0);
        } 
    } 

}

