package ejava.util.ant;

import java.io.*;
import java.util.*;

import java.util.prefs.*;

import org.apache.tools.ant.*;

import ejava.util.prefs.*;

public class PreferenceSetterTask extends Task 
{
    private String root = "user";     // should be "system" or "user"
    private String node = "";     // relative path of node
    private File importFile = null; // XML file to import from, overrides previous two
    private File exportFile = null; // XML file to write out to
    private Vector entries = new Vector();

    private PreferenceSetter ps;

    public PreferenceSetterTask()
    {
	ps = new PreferenceSetter();
    }

    public void setRoot(String root)
    {
	this.root = root;
    }

    public void setNode(String node)
    {
	this.node = node;
    }

    public void setImportFile(File file)
    {
	this.importFile = file;
    }

    public void setExportFile(File file)
    {
	this.exportFile = file;
    }

    public Entry createEntry()
    {
	Entry e = new Entry();
	entries.addElement(e);
	return e;
    }

    public void execute() throws BuildException
    {
	if (importFile != null)
	    executeImportFile();
	else
	    executeNodeEntries();

	if (exportFile != null)
	    executeExportFile();
    }
    
    private void executeImportFile() throws BuildException
    {
	try
	{
	    FileInputStream fis = new FileInputStream(importFile);
	    ps.importPreferences(fis);
	}
	catch (Exception e)
	{
	    throw new BuildException(e);
	}
    }

    private void executeNodeEntries() throws BuildException
    {
        if (root.equalsIgnoreCase("system"))
	    ps.setSystemRoot();
	else if (root.equalsIgnoreCase("user"))
	    ps.setUserRoot();
	else
	    throw new BuildException("root attribute must be 'system' or 'user'");

	ps.setNode(node);
	Iterator it = entries.iterator();
	while (it.hasNext())
	{
	    Entry entry = (Entry)it.next();
	    ps.addNodeEntry(entry.getKey(), entry.getValue());
	}
    }

    private void executeExportFile() throws BuildException
    {
	try
	{
	    FileOutputStream fos = new FileOutputStream(exportFile);
	    ps.exportPreferences(fos);
	}
	catch (Exception e)
	{
	    throw new BuildException(e);
	}
    }

    public static class Entry
    {
	private String key;
	private String value;

	public void setKey(String keyStr)
	{
	    this.key = keyStr;
	}

	public void setValue(String valueStr)
	{
	    this.value = valueStr;
	}

	public String getKey()
	{
	    return this.key;
	}

	public String getValue()
	{
	    return this.value;
	}
    }
}
