package ejava.webservices;

/**
 * @author Perrone, Chaganti, and Schwenk
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices
 * 
 * Type: DatabaseHelper
 * 
 */


import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.DriverManager;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import java.util.prefs.Preferences;

public class DatabaseHelper {
    private Connection          connection = null;


    public DatabaseHelper(){
    }

    /**
     * To load the driver and make connection to the DB.
     */
    public void makeDbConnection(){
        try{
            Preferences prefs =
                Preferences.userNodeForPackage(DatabaseHelper.class);
    
            // Get driver class, database URL, username, and password,
            // and then create DB Connection with this info
            String      driverClass = prefs.get("DRIVER_CLASS", null);
    
            Class.forName(driverClass);
    
            String dbUrl = prefs.get("DATABASE_URL", null);
            String userName = prefs.get("USER_NAME", null);
            String password = prefs.get("PASSWORD", null);
    
            if (userName == null) {
                connection = DriverManager.getConnection(dbUrl);
            } else {
                connection = DriverManager.getConnection(dbUrl, userName,
                                                        password);
            }
        }
        catch(Exception e){
            e.printStackTrace();
        }
    }

    public synchronized ResultSet getResultSet(String sqlQuery, Object whereValue)
            throws SQLException {

        if(connection==null){
            this.makeDbConnection();
        }
        // Create prepared statement, set where value, execute query
        PreparedStatement preparedStatement =
            connection.prepareStatement(sqlQuery);

        preparedStatement.setObject(1, whereValue);

        ResultSet rs = preparedStatement.executeQuery();

        return rs;
    }
    
    public synchronized ResultSet getResultSet(String sqlQuery)
            throws SQLException {

        if(connection==null){
            this.makeDbConnection();
        }

        Statement statement = connection.createStatement();
        
        ResultSet rs = statement.executeQuery(sqlQuery);

        return rs;
    }
    
}

