package ejava.webservices;

/**
 * @author pperrone
 *
 * J2EE Developer's Handbook
 */

import javax.xml.soap.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.util.*;

public class SOAPMessageHelper {

    /**
     * Dump the guts of the SOAP Message to screen and return the
     * size of the message.
     */
    public static int printMessage(SOAPMessage msg) 
                throws SOAPException, IOException{        
        FileOutputStream fos = new FileOutputStream(FileDescriptor.out);            
        DataOutputStream dos = new DataOutputStream(fos);                        
        msg.writeTo(dos);
        int size = dos.size();
        System.out.println("\nSOAP Message above is of size: " + size +"\n");
        fos.flush();
        return size;
    }

    /**
     * Dump the guts of the SOAP Message to screen
     */
    public static void printMessage(InputStream msg) 
                throws SOAPException, IOException{
        int bytes = msg.available();
        byte byteStream[] = new byte[bytes];
        msg.read(byteStream);
        FileOutputStream fos = new FileOutputStream(FileDescriptor.out);
        fos.write(byteStream);
    }

    /**
     * Strip off HTTP header and return SOAP String
     */
    public static String parseSOAPString(InputStream msg){
        String soapString = "";
        try{
            int bytes = msg.available();
            byte byteStream[] = new byte[bytes];
            msg.read(byteStream);
            String str = new String(byteStream);
            System.out.println("Full Message Format: \n"+str);
            soapString = str.substring(str.indexOf("<?xml"));
        }
        catch(Exception e){
            e.printStackTrace();
        }
        return soapString;
    }
           
    /**
     * Create a SOAPMessage from a HashMap
     */
    public static SOAPMessage createMessage(HashMap hashMessage){ 
        SOAPMessage msg = null;
        try{
          // Create a new empty SOAPMessage to hold the response
            Document doc 
                = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
            MessageFactory msgFactory = MessageFactory.newInstance();                

            SOAPFactory soapF = SOAPFactory.newInstance();
            msg = msgFactory.createMessage();
            SOAPPart part = msg.getSOAPPart();
            SOAPEnvelope env = part.getEnvelope();
            SOAPBody body = env.getBody();
               
            //Name name = env.createName("findCustomersResponse", "ejava.webservices.Customer", "http://beeshirts.com/soap");
            //SOAPElement e1 = body.addChildElement(name);
            //SOAPElement e2 = e1.addChildElement("minimumOrders");
            //SOAPElement e3 = e2.addTextNode("313");

            // Systematically put HashMap response values into SOAP message
            Iterator names = hashMessage.keySet().iterator();
            while(names.hasNext()){
                String objName = (String) names.next();
                Object returnObject = hashMessage.get(objName);
                if(returnObject instanceof HashMap){
                    insertElement((HashMap)returnObject, body);                
                }
                else if(returnObject instanceof Collection){
                    Iterator elements = ((Collection) returnObject).iterator();
                    for(int i = 0; elements.hasNext(); ++i){
                        Name element = soapF.createName(objName+":"+i);
                        SOAPElement bodyChild = body.addBodyElement(element);                    
                        HashMap subElement = (HashMap) elements.next();
                        insertElement(subElement, bodyChild);
                    }
                }                
            }
            
            msg.saveChanges();
        }
        catch(Exception e){
            e.printStackTrace();
        }
        
        return msg;
    }
    
    /**
     * Insert data from the HashMap into the SOAPElement
     */
    public static void insertElement(HashMap from, SOAPElement to){
        try{
            Iterator names = from.keySet().iterator();
            while(names.hasNext()){
                String objName = (String) names.next();
                String returnObject = (String) from.get(objName);
                if(returnObject != null){
                    Name element = SOAPFactory.newInstance().createName(objName);
                    SOAPElement child = to.addChildElement(element);
                    child.addTextNode(returnObject);
                }
            }
        }
        catch(Exception e){
            e.printStackTrace();
        }
        
    }
    
    /**
     * This method parses a SOAPString into a HashMap representation
     */
    public static HashMap parseMessage(String soapString){
        HashMap hashMessage = new HashMap();
        try{
            // First create a byte array stream out of string
            byte byteStream[] = soapString.getBytes();
            ByteArrayInputStream bais 
                = new ByteArrayInputStream(byteStream);
            
            // Parse input stream into a DOM document
            DocumentBuilder docBuilder 
                = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document soapDoc = docBuilder.parse(bais);

            // Get to the request data
            org.w3c.dom.Node envelope = soapDoc.getFirstChild();
            org.w3c.dom.Node body = envelope.getFirstChild().getNextSibling();
            org.w3c.dom.Node request = body.getFirstChild();

            // Get parameters from the SOAP body request
            // and put inside the HashMap
            NodeList nl = request.getChildNodes();
            for(int i = 0; i < nl.getLength(); ++i){
                String name = nl.item(i).getNodeName();
                String value = nl.item(i).getFirstChild().getNodeValue();
                hashMessage.put(name, value);
            }
            
            // Also put the class and method type inside hashmap
            String requestType = request.getNodeName();
            int index = requestType.indexOf(":");
            String classType = requestType.substring(0,index);
            hashMessage.put("beeshirts:CLASS_NAME", classType);
            String methodType = requestType.substring(index+1);
            hashMessage.put("beeshirts:METHOD_NAME", methodType);
        }
        catch(Exception e){
            e.printStackTrace();
        }
        
        return hashMessage;
    }

}
