package ejava.webservices.rpc;

/**
 * @author Paul Perrone
 *
 * J2EE Developer's Handbook
 * 
 * Package: ejava.webservices.rpc
 * 
 * Type: LoginHandler
 * 
 * Implements a security handler on the client side.
 * 
 */

import javax.xml.rpc.handler.*;
import javax.xml.rpc.handler.soap.*;
import javax.xml.soap.*;
import java.util.Iterator;
import java.util.prefs.Preferences;

public class LoginHandler extends SecurityHandler {

  public LoginHandler(){ 
  }

  public void init(HandlerInfo config){
  	// Read parent security header name
    super.init(config);
  }
  
  public boolean handleRequest(MessageContext ctx){
  	// Handle the request by appending the username/password
  	//    into the SOAP header as read from preferences
    boolean status = false;
    try{
      System.out.println("  -- HANDLE REQUEST ON CLIENT --");
    	
   	  // Get client username and password from JVM properties
      String username = System.getProperty("USERNAME", "");
      String password = System.getProperty("PASSWORD", "");
      
      // Get SOAP Message Context and Envelope
      SOAPMessageContext soapCtx = (SOAPMessageContext) ctx;
      SOAPMessage message = soapCtx.getMessage();
      SOAPPart part = message.getSOAPPart();
      SOAPEnvelope envelope = part.getEnvelope();
      
      // Get/add SOAP header
      SOAPHeader header = envelope.getHeader();
      if(header == null){
        header = envelope.addHeader();
      }
      
      // Add SOAP header element security info
      SOAPElement securityElement
        = header.addChildElement(NAME, PREFIX, NAMESPACE);
      securityElement.addChildElement(USERNAME).addTextNode(username);
      securityElement.addChildElement(PASSWORD).addTextNode(password);

      status = true;
    }
    catch(Exception e){
    	System.out.println("LoginHandler error: " 
    						+ e.getMessage());
    }
    return status;
  }  
}
